#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""End to end tests for ProtoRPC."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import unittest

from protorpc import protojson
from protorpc import remote
from protorpc import test_util
from protorpc import util
from protorpc import webapp_test_util

package = 'test_package'


class EndToEndTest(webapp_test_util.EndToEndTestBase):

  def testSimpleRequest(self):
    self.assertEquals(test_util.OptionalMessage(string_value='+blar'),
                      self.stub.optional_message(string_value='blar'))

  def testSimpleRequestComplexContentType(self):
    response = self.DoRawRequest(
      'optional_message',
      content='{"string_value": "blar"}',
      content_type='application/json; charset=utf-8')
    headers = response.headers
    self.assertEquals(200, response.code)
    self.assertEquals('{"string_value": "+blar"}', response.read())
    self.assertEquals('application/json', headers['content-type'])

  def testInitParameter(self):
    self.assertEquals(test_util.OptionalMessage(string_value='uninitialized'),
                      self.stub.init_parameter())
    self.assertEquals(test_util.OptionalMessage(string_value='initialized'),
                      self.other_stub.init_parameter())

  def testMissingContentType(self):
    code, content, headers = self.RawRequestError(
      'optional_message',
      content='{"string_value": "blar"}',
      content_type='')
    self.assertEquals(400, code)
    self.assertEquals(util.pad_string('Bad Request'), content)
    self.assertEquals('text/plain; charset=utf-8', headers['content-type'])

  def testWrongPath(self):
    self.assertRaisesWithRegexpMatch(remote.ServerError,
                                     'HTTP Error 404: Not Found',
                                     self.bad_path_stub.optional_message)

  def testUnsupportedContentType(self):
    code, content, headers = self.RawRequestError(
      'optional_message',
      content='{"string_value": "blar"}',
      content_type='image/png')
    self.assertEquals(415, code)
    self.assertEquals(util.pad_string('Unsupported Media Type'), content)
    self.assertEquals(headers['content-type'], 'text/plain; charset=utf-8')

  def testUnsupportedHttpMethod(self):
    code, content, headers = self.RawRequestError('optional_message')
    self.assertEquals(405, code)
    self.assertEquals(
      util.pad_string('/my/service.optional_message is a ProtoRPC method.\n\n'
                      'Service protorpc.webapp_test_util.TestService\n\n'
                      'More about ProtoRPC: '
                      'http://code.google.com/p/google-protorpc\n'),
      content)
    self.assertEquals(headers['content-type'], 'text/plain; charset=utf-8')

  def testMethodNotFound(self):
    self.assertRaisesWithRegexpMatch(remote.MethodNotFoundError,
                                     'Unrecognized RPC method: does_not_exist',
                                     self.mismatched_stub.does_not_exist)

  def testBadMessageError(self):
    code, content, headers = self.RawRequestError('nested_message',
                                                  content='{}')
    self.assertEquals(400, code)

    expected_content = protojson.encode_message(remote.RpcStatus(
      state=remote.RpcState.REQUEST_ERROR,
      error_message=('Error parsing ProtoRPC request '
                     '(Unable to parse request content: '
                     'Message NestedMessage is missing '
                     'required field a_value)')))
    self.assertEquals(util.pad_string(expected_content), content)
    self.assertEquals(headers['content-type'], 'application/json')

  def testApplicationError(self):
    try:
      self.stub.raise_application_error()
    except remote.ApplicationError as err:
      self.assertEquals('This is an application error', unicode(err))
      self.assertEquals('ERROR_NAME', err.error_name)
    else:
      self.fail('Expected application error')

  def testRpcError(self):
    try:
      self.stub.raise_rpc_error()
    except remote.ServerError as err:
      self.assertEquals('Internal Server Error', unicode(err))
    else:
      self.fail('Expected server error')

  def testUnexpectedError(self):
    try:
      self.stub.raise_unexpected_error()
    except remote.ServerError as err:
      self.assertEquals('Internal Server Error', unicode(err))
    else:
      self.fail('Expected server error')

  def testBadResponse(self):
    try:
      self.stub.return_bad_message()
    except remote.ServerError as err:
      self.assertEquals('Internal Server Error', unicode(err))
    else:
      self.fail('Expected server error')


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.descriptor."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import types
import unittest

from protorpc import descriptor
from protorpc import message_types
from protorpc import messages
from protorpc import registry
from protorpc import remote
from protorpc import test_util


RUSSIA = u'\u0420\u043e\u0441\u0441\u0438\u044f'


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = descriptor


class DescribeEnumValueTest(test_util.TestCase):

  def testDescribe(self):
    class MyEnum(messages.Enum):
      MY_NAME = 10

    expected = descriptor.EnumValueDescriptor()
    expected.name = 'MY_NAME'
    expected.number = 10

    described = descriptor.describe_enum_value(MyEnum.MY_NAME)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeEnumTest(test_util.TestCase):

  def testEmptyEnum(self):
    class EmptyEnum(messages.Enum):
      pass

    expected = descriptor.EnumDescriptor()
    expected.name = 'EmptyEnum'

    described = descriptor.describe_enum(EmptyEnum)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testNestedEnum(self):
    class MyScope(messages.Message):
      class NestedEnum(messages.Enum):
        pass

    expected = descriptor.EnumDescriptor()
    expected.name = 'NestedEnum'

    described = descriptor.describe_enum(MyScope.NestedEnum)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testEnumWithItems(self):
    class EnumWithItems(messages.Enum):
      A = 3
      B = 1
      C = 2

    expected = descriptor.EnumDescriptor()
    expected.name = 'EnumWithItems'

    a = descriptor.EnumValueDescriptor()
    a.name = 'A'
    a.number = 3

    b = descriptor.EnumValueDescriptor()
    b.name = 'B'
    b.number = 1

    c = descriptor.EnumValueDescriptor()
    c.name = 'C'
    c.number = 2

    expected.values = [b, c, a]

    described = descriptor.describe_enum(EnumWithItems)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeFieldTest(test_util.TestCase):

  def testLabel(self):
    for repeated, required, expected_label in (
        (True, False, descriptor.FieldDescriptor.Label.REPEATED),
        (False, True, descriptor.FieldDescriptor.Label.REQUIRED),
        (False, False, descriptor.FieldDescriptor.Label.OPTIONAL)):
      field = messages.IntegerField(10, required=required, repeated=repeated)
      field.name = 'a_field'

      expected = descriptor.FieldDescriptor()
      expected.name = 'a_field'
      expected.number = 10
      expected.label = expected_label
      expected.variant = descriptor.FieldDescriptor.Variant.INT64

      described = descriptor.describe_field(field)
      described.check_initialized()
      self.assertEquals(expected, described)

  def testDefault(self):
    for field_class, default, expected_default in (
        (messages.IntegerField, 200, '200'),
        (messages.FloatField, 1.5, '1.5'),
        (messages.FloatField, 1e6, '1000000.0'),
        (messages.BooleanField, True, 'true'),
        (messages.BooleanField, False, 'false'),
        (messages.BytesField, 'ab\xF1', 'ab\\xf1'),
        (messages.StringField, RUSSIA, RUSSIA),
        ):
      field = field_class(10, default=default)
      field.name = u'a_field'

      expected = descriptor.FieldDescriptor()
      expected.name = u'a_field'
      expected.number = 10
      expected.label = descriptor.FieldDescriptor.Label.OPTIONAL
      expected.variant = field_class.DEFAULT_VARIANT
      expected.default_value = expected_default

      described = descriptor.describe_field(field)
      described.check_initialized()
      self.assertEquals(expected, described)

  def testDefault_EnumField(self):
    class MyEnum(messages.Enum):

      VAL = 1

    module_name = test_util.get_module_name(MyEnum)
    field = messages.EnumField(MyEnum, 10, default=MyEnum.VAL)
    field.name = 'a_field'

    expected = descriptor.FieldDescriptor()
    expected.name = 'a_field'
    expected.number = 10
    expected.label = descriptor.FieldDescriptor.Label.OPTIONAL
    expected.variant = messages.EnumField.DEFAULT_VARIANT
    expected.type_name = '%s.MyEnum' % module_name
    expected.default_value = '1'

    described = descriptor.describe_field(field)
    self.assertEquals(expected, described)

  def testMessageField(self):
    field = messages.MessageField(descriptor.FieldDescriptor, 10)
    field.name = 'a_field'

    expected = descriptor.FieldDescriptor()
    expected.name = 'a_field'
    expected.number = 10
    expected.label = descriptor.FieldDescriptor.Label.OPTIONAL
    expected.variant = messages.MessageField.DEFAULT_VARIANT
    expected.type_name = ('protorpc.descriptor.FieldDescriptor')

    described = descriptor.describe_field(field)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testDateTimeField(self):
    field = message_types.DateTimeField(20)
    field.name = 'a_timestamp'

    expected = descriptor.FieldDescriptor()
    expected.name = 'a_timestamp'
    expected.number = 20
    expected.label = descriptor.FieldDescriptor.Label.OPTIONAL
    expected.variant = messages.MessageField.DEFAULT_VARIANT
    expected.type_name = ('protorpc.message_types.DateTimeMessage')

    described = descriptor.describe_field(field)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeMessageTest(test_util.TestCase):

  def testEmptyDefinition(self):
    class MyMessage(messages.Message):
      pass

    expected = descriptor.MessageDescriptor()
    expected.name = 'MyMessage'

    described = descriptor.describe_message(MyMessage)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testDefinitionWithFields(self):
    class MessageWithFields(messages.Message):
      field1 = messages.IntegerField(10)
      field2 = messages.StringField(30)
      field3 = messages.IntegerField(20)

    expected = descriptor.MessageDescriptor()
    expected.name = 'MessageWithFields'

    expected.fields = [
      descriptor.describe_field(MessageWithFields.field_by_name('field1')),
      descriptor.describe_field(MessageWithFields.field_by_name('field3')),
      descriptor.describe_field(MessageWithFields.field_by_name('field2')),
    ]

    described = descriptor.describe_message(MessageWithFields)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testNestedEnum(self):
    class MessageWithEnum(messages.Message):
      class Mood(messages.Enum):
        GOOD = 1
        BAD = 2
        UGLY = 3

      class Music(messages.Enum):
        CLASSIC = 1
        JAZZ = 2
        BLUES = 3

    expected = descriptor.MessageDescriptor()
    expected.name = 'MessageWithEnum'

    expected.enum_types = [descriptor.describe_enum(MessageWithEnum.Mood),
                           descriptor.describe_enum(MessageWithEnum.Music)]

    described = descriptor.describe_message(MessageWithEnum)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testNestedMessage(self):
    class MessageWithMessage(messages.Message):
      class Nesty(messages.Message):
        pass

    expected = descriptor.MessageDescriptor()
    expected.name = 'MessageWithMessage'

    expected.message_types = [
      descriptor.describe_message(MessageWithMessage.Nesty)]

    described = descriptor.describe_message(MessageWithMessage)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeMethodTest(test_util.TestCase):
  """Test describing remote methods."""

  def testDescribe(self):
    class Request(messages.Message):
      pass

    class Response(messages.Message):
      pass

    @remote.method(Request, Response)
    def remote_method(request):
      pass

    module_name = test_util.get_module_name(DescribeMethodTest)
    expected = descriptor.MethodDescriptor()
    expected.name = 'remote_method'
    expected.request_type = '%s.Request' % module_name
    expected.response_type = '%s.Response' % module_name

    described = descriptor.describe_method(remote_method)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeServiceTest(test_util.TestCase):
  """Test describing service classes."""

  def testDescribe(self):
    class Request1(messages.Message):
      pass

    class Response1(messages.Message):
      pass

    class Request2(messages.Message):
      pass

    class Response2(messages.Message):
      pass

    class MyService(remote.Service):

      @remote.method(Request1, Response1)
      def method1(self, request):
        pass

      @remote.method(Request2, Response2)
      def method2(self, request):
        pass

    expected = descriptor.ServiceDescriptor()
    expected.name = 'MyService'
    expected.methods = []

    expected.methods.append(descriptor.describe_method(MyService.method1))
    expected.methods.append(descriptor.describe_method(MyService.method2))

    described = descriptor.describe_service(MyService)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeFileTest(test_util.TestCase):
  """Test describing modules."""

  def LoadModule(self, module_name, source):
    result = {'__name__': module_name,
              'messages': messages,
              'remote': remote,
              }
    exec(source, result)

    module = types.ModuleType(module_name)
    for name, value in result.items():
      setattr(module, name, value)

    return module

  def testEmptyModule(self):
    """Test describing an empty file."""
    module = types.ModuleType('my.package.name')

    expected = descriptor.FileDescriptor()
    expected.package = 'my.package.name'

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testNoPackageName(self):
    """Test describing a module with no module name."""
    module = types.ModuleType('')

    expected = descriptor.FileDescriptor()

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testPackageName(self):
    """Test using the 'package' module attribute."""
    module = types.ModuleType('my.module.name')
    module.package = 'my.package.name'

    expected = descriptor.FileDescriptor()
    expected.package = 'my.package.name'

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testMain(self):
    """Test using the 'package' module attribute."""
    module = types.ModuleType('__main__')
    module.__file__ = '/blim/blam/bloom/my_package.py'

    expected = descriptor.FileDescriptor()
    expected.package = 'my_package'

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testMessages(self):
    """Test that messages are described."""
    module = self.LoadModule('my.package',
                             'class Message1(messages.Message): pass\n'
                             'class Message2(messages.Message): pass\n')

    message1 = descriptor.MessageDescriptor()
    message1.name = 'Message1'

    message2 = descriptor.MessageDescriptor()
    message2.name = 'Message2'

    expected = descriptor.FileDescriptor()
    expected.package = 'my.package'
    expected.message_types = [message1, message2]

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testEnums(self):
    """Test that enums are described."""
    module = self.LoadModule('my.package',
                             'class Enum1(messages.Enum): pass\n'
                             'class Enum2(messages.Enum): pass\n')

    enum1 = descriptor.EnumDescriptor()
    enum1.name = 'Enum1'

    enum2 = descriptor.EnumDescriptor()
    enum2.name = 'Enum2'

    expected = descriptor.FileDescriptor()
    expected.package = 'my.package'
    expected.enum_types = [enum1, enum2]

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)

  def testServices(self):
    """Test that services are described."""
    module = self.LoadModule('my.package',
                             'class Service1(remote.Service): pass\n'
                             'class Service2(remote.Service): pass\n')

    service1 = descriptor.ServiceDescriptor()
    service1.name = 'Service1'

    service2 = descriptor.ServiceDescriptor()
    service2.name = 'Service2'

    expected = descriptor.FileDescriptor()
    expected.package = 'my.package'
    expected.service_types = [service1, service2]

    described = descriptor.describe_file(module)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeFileSetTest(test_util.TestCase):
  """Test describing multiple modules."""

  def testNoModules(self):
    """Test what happens when no modules provided."""
    described = descriptor.describe_file_set([])
    described.check_initialized()
    # The described FileSet.files will be None.
    self.assertEquals(descriptor.FileSet(), described)

  def testWithModules(self):
    """Test what happens when no modules provided."""
    modules = [types.ModuleType('package1'), types.ModuleType('package1')]

    file1 = descriptor.FileDescriptor()
    file1.package = 'package1'
    file2 = descriptor.FileDescriptor()
    file2.package = 'package2'

    expected = descriptor.FileSet()
    expected.files = [file1, file1]

    described = descriptor.describe_file_set(modules)
    described.check_initialized()
    self.assertEquals(expected, described)


class DescribeTest(test_util.TestCase):

  def testModule(self):
    self.assertEquals(descriptor.describe_file(test_util),
                      descriptor.describe(test_util))

  def testMethod(self):
    class Param(messages.Message):
      pass

    class Service(remote.Service):

      @remote.method(Param, Param)
      def fn(self):
        return Param()

    self.assertEquals(descriptor.describe_method(Service.fn),
                      descriptor.describe(Service.fn))

  def testField(self):
    self.assertEquals(
      descriptor.describe_field(test_util.NestedMessage.a_value),
      descriptor.describe(test_util.NestedMessage.a_value))

  def testEnumValue(self):
    self.assertEquals(
      descriptor.describe_enum_value(
        test_util.OptionalMessage.SimpleEnum.VAL1),
      descriptor.describe(test_util.OptionalMessage.SimpleEnum.VAL1))

  def testMessage(self):
    self.assertEquals(descriptor.describe_message(test_util.NestedMessage),
                      descriptor.describe(test_util.NestedMessage))

  def testEnum(self):
    self.assertEquals(
      descriptor.describe_enum(test_util.OptionalMessage.SimpleEnum),
      descriptor.describe(test_util.OptionalMessage.SimpleEnum))

  def testService(self):
    class Service(remote.Service):
      pass

    self.assertEquals(descriptor.describe_service(Service),
                      descriptor.describe(Service))

  def testService(self):
    class Service(remote.Service):
      pass

    self.assertEquals(descriptor.describe_service(Service),
                      descriptor.describe(Service))

  def testUndescribable(self):
    class NonService(object):

      def fn(self):
        pass

    for value in (NonService,
                  NonService.fn,
                  1,
                  'string',
                  1.2,
                  None):
      self.assertEquals(None, descriptor.describe(value))


class ModuleFinderTest(test_util.TestCase):

  def testFindModule(self):
    self.assertEquals(descriptor.describe_file(registry),
                      descriptor.import_descriptor_loader('protorpc.registry'))

  def testFindMessage(self):
    self.assertEquals(
      descriptor.describe_message(descriptor.FileSet),
      descriptor.import_descriptor_loader('protorpc.descriptor.FileSet'))

  def testFindField(self):
    self.assertEquals(
      descriptor.describe_field(descriptor.FileSet.files),
      descriptor.import_descriptor_loader('protorpc.descriptor.FileSet.files'))

  def testFindEnumValue(self):
    self.assertEquals(
      descriptor.describe_enum_value(test_util.OptionalMessage.SimpleEnum.VAL1),
      descriptor.import_descriptor_loader(
        'protorpc.test_util.OptionalMessage.SimpleEnum.VAL1'))

  def testFindMethod(self):
    self.assertEquals(
      descriptor.describe_method(registry.RegistryService.services),
      descriptor.import_descriptor_loader(
        'protorpc.registry.RegistryService.services'))

  def testFindService(self):
    self.assertEquals(
      descriptor.describe_service(registry.RegistryService),
      descriptor.import_descriptor_loader('protorpc.registry.RegistryService'))

  def testFindWithAbsoluteName(self):
    self.assertEquals(
      descriptor.describe_service(registry.RegistryService),
      descriptor.import_descriptor_loader('.protorpc.registry.RegistryService'))

  def testFindWrongThings(self):
    for name in ('a', 'protorpc.registry.RegistryService.__init__', '', ):
      self.assertRaisesWithRegexpMatch(
        messages.DefinitionNotFoundError,
        'Could not find definition for %s' % name,
        descriptor.import_descriptor_loader, name)


class DescriptorLibraryTest(test_util.TestCase):

  def setUp(self):
    self.packageless = descriptor.MessageDescriptor()
    self.packageless.name = 'Packageless'
    self.library = descriptor.DescriptorLibrary(
      descriptors={
        'not.real.Packageless': self.packageless,
        'Packageless': self.packageless,
      })

  def testLookupPackage(self):
    self.assertEquals('csv', self.library.lookup_package('csv'))
    self.assertEquals('protorpc', self.library.lookup_package('protorpc'))
    self.assertEquals('protorpc.registry',
                      self.library.lookup_package('protorpc.registry'))
    self.assertEquals('protorpc.registry',
                      self.library.lookup_package('.protorpc.registry'))
    self.assertEquals(
      'protorpc.registry',
      self.library.lookup_package('protorpc.registry.RegistryService'))
    self.assertEquals(
      'protorpc.registry',
      self.library.lookup_package(
        'protorpc.registry.RegistryService.services'))

  def testLookupNonPackages(self):
    for name in ('', 'a', 'protorpc.descriptor.DescriptorLibrary'):
      self.assertRaisesWithRegexpMatch(
        messages.DefinitionNotFoundError,
        'Could not find definition for %s' % name,
        self.library.lookup_package, name)

  def testNoPackage(self):
    self.assertRaisesWithRegexpMatch(
      messages.DefinitionNotFoundError,
      'Could not find definition for not.real',
      self.library.lookup_package, 'not.real.Packageless')

    self.assertEquals(None, self.library.lookup_package('Packageless'))


def main():
  unittest.main()


if __name__ == '__main__':
  main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Service regsitry for service discovery.

The registry service can be deployed on a server in order to provide a
central place where remote clients can discover available.

On the server side, each service is registered by their name which is unique
to the registry.  Typically this name provides enough information to identify
the service and locate it within a server.  For example, for an HTTP based
registry the name is the URL path on the host where the service is invocable.

The registry is also able to resolve the full descriptor.FileSet necessary to
describe the service and all required data-types (messages and enums).

A configured registry is itself a remote service and should reference itself.
"""

import sys

from . import descriptor
from . import messages
from . import remote
from . import util


__all__ = [
  'ServiceMapping',
  'ServicesResponse',
  'GetFileSetRequest',
  'GetFileSetResponse',
  'RegistryService',
]


class ServiceMapping(messages.Message):
  """Description of registered service.

  Fields:
    name: Name of service.  On HTTP based services this will be the
      URL path used for invocation.
    definition: Fully qualified name of the service definition.  Useful
      for clients that can look up service definitions based on an existing
      repository of definitions.
  """

  name = messages.StringField(1, required=True)
  definition = messages.StringField(2, required=True)


class ServicesResponse(messages.Message):
  """Response containing all registered services.

  May also contain complete descriptor file-set for all services known by the
  registry.

  Fields:
    services: Service mappings for all registered services in registry.
    file_set: Descriptor file-set describing all services, messages and enum
      types needed for use with all requested services if asked for in the
      request.
  """

  services = messages.MessageField(ServiceMapping, 1, repeated=True)


class GetFileSetRequest(messages.Message):
  """Request for service descriptor file-set.

  Request to retrieve file sets for specific services.

  Fields:
    names: Names of services to retrieve file-set for.
  """

  names = messages.StringField(1, repeated=True)


class GetFileSetResponse(messages.Message):
  """Descriptor file-set for all names in GetFileSetRequest.

  Fields:
    file_set: Descriptor file-set containing all descriptors for services,
      messages and enum types needed for listed names in request.
  """

  file_set = messages.MessageField(descriptor.FileSet, 1, required=True)


class RegistryService(remote.Service):
  """Registry service.

  Maps names to services and is able to describe all descriptor file-sets
  necessary to use contined services.

  On an HTTP based server, the name is the URL path to the service.
  """

  @util.positional(2)
  def __init__(self, registry, modules=None):
    """Constructor.

    Args:
      registry: Map of name to service class.  This map is not copied and may
        be modified after the reigstry service has been configured.
      modules: Module dict to draw descriptors from.  Defaults to sys.modules.
    """
    # Private Attributes:
    #   __registry: Map of name to service class.  Refers to same instance as
    #     registry parameter.
    #   __modules: Mapping of module name to module.
    #   __definition_to_modules: Mapping of definition types to set of modules
    #     that they refer to.  This cache is used to make repeated look-ups
    #     faster and to prevent circular references from causing endless loops.

    self.__registry = registry
    if modules is None:
      modules = sys.modules
    self.__modules = modules
    # This cache will only last for a single request.
    self.__definition_to_modules = {}

  def __find_modules_for_message(self, message_type):
    """Find modules referred to by a message type.

    Determines the entire list of modules ultimately referred to by message_type
    by iterating over all of its message and enum fields.  Includes modules
    referred to fields within its referred messages.

    Args:
      message_type: Message type to find all referring modules for.

    Returns:
      Set of modules referred to by message_type by traversing all its
      message and enum fields.
    """
    # TODO(rafek): Maybe this should be a method on Message and Service?
    def get_dependencies(message_type, seen=None):
      """Get all dependency definitions of a message type.

      This function works by collecting the types of all enumeration and message
      fields defined within the message type.  When encountering a message
      field, it will recursivly find all of the associated message's
      dependencies.  It will terminate on circular dependencies by keeping track
      of what definitions it already via the seen set.

      Args:
        message_type: Message type to get dependencies for.
        seen: Set of definitions that have already been visited.

      Returns:
        All dependency message and enumerated types associated with this message
        including the message itself.
      """
      if seen is None:
        seen = set()
      seen.add(message_type)

      for field in message_type.all_fields():
        if isinstance(field, messages.MessageField):
          if field.message_type not in seen:
            get_dependencies(field.message_type, seen)
        elif isinstance(field, messages.EnumField):
          seen.add(field.type)

      return seen

    found_modules = self.__definition_to_modules.setdefault(message_type, set())
    if not found_modules:
      dependencies = get_dependencies(message_type)
      found_modules.update(self.__modules[definition.__module__]
                           for definition in dependencies)

    return found_modules

  def __describe_file_set(self, names):
    """Get file-set for named services.

    Args:
      names: List of names to get file-set for.

    Returns:
      descriptor.FileSet containing all the descriptors for all modules
      ultimately referred to by all service types request by names parameter.
    """
    service_modules = set()
    if names:
      for service in (self.__registry[name] for name in names):
        found_modules = self.__definition_to_modules.setdefault(service, set())
        if not found_modules:
          found_modules.add(self.__modules[service.__module__])
          for method_name in service.all_remote_methods():
            method = getattr(service, method_name)
            for message_type in (method.remote.request_type,
                                 method.remote.response_type):
              found_modules.update(
                self.__find_modules_for_message(message_type))
        service_modules.update(found_modules)

    return descriptor.describe_file_set(service_modules)

  @property
  def registry(self):
    """Get service registry associated with this service instance."""
    return self.__registry

  @remote.method(response_type=ServicesResponse)
  def services(self, request):
    """Get all registered services."""
    response = ServicesResponse()
    response.services = []
    for name, service_class in self.__registry.items():
      mapping = ServiceMapping()
      mapping.name = name.decode('utf-8')
      mapping.definition = service_class.definition_name().decode('utf-8')
      response.services.append(mapping)

    return response

  @remote.method(GetFileSetRequest, GetFileSetResponse)
  def get_file_set(self, request):
    """Get file-set for registered servies."""
    response = GetFileSetResponse()
    response.file_set = self.__describe_file_set(request.names)
    return response



#!/usr/bin/env python
#
# Copyright 2013 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.message_types."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import datetime

import unittest

from protorpc import message_types
from protorpc import messages
from protorpc import test_util
from protorpc import util


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = message_types


class DateTimeFieldTest(test_util.TestCase):

  def testValueToMessage(self):
    field = message_types.DateTimeField(1)
    message = field.value_to_message(datetime.datetime(2033, 2, 4, 11, 22, 10))
    self.assertEqual(message_types.DateTimeMessage(milliseconds=1991128930000),
                     message)

  def testValueToMessageBadValue(self):
    field = message_types.DateTimeField(1)
    self.assertRaisesWithRegexpMatch(
        messages.EncodeError,
        'Expected type datetime, got int: 20',
        field.value_to_message, 20)

  def testValueToMessageWithTimeZone(self):
    time_zone = util.TimeZoneOffset(60 * 10)
    field = message_types.DateTimeField(1)
    message = field.value_to_message(
        datetime.datetime(2033, 2, 4, 11, 22, 10, tzinfo=time_zone))
    self.assertEqual(message_types.DateTimeMessage(milliseconds=1991128930000,
                                                   time_zone_offset=600),
                     message)

  def testValueFromMessage(self):
    message = message_types.DateTimeMessage(milliseconds=1991128000000)
    field = message_types.DateTimeField(1)
    timestamp = field.value_from_message(message)
    self.assertEqual(datetime.datetime(2033, 2, 4, 11, 6, 40),
                     timestamp)

  def testValueFromMessageBadValue(self):
    field = message_types.DateTimeField(1)
    self.assertRaisesWithRegexpMatch(
        messages.DecodeError,
        'Expected type DateTimeMessage, got VoidMessage: <VoidMessage>',
        field.value_from_message, message_types.VoidMessage())

  def testValueFromMessageWithTimeZone(self):
    message = message_types.DateTimeMessage(milliseconds=1991128000000,
                                            time_zone_offset=300)
    field = message_types.DateTimeField(1)
    timestamp = field.value_from_message(message)
    time_zone = util.TimeZoneOffset(60 * 5)
    self.assertEqual(datetime.datetime(2033, 2, 4, 11, 6, 40, tzinfo=time_zone),
                     timestamp)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Webapp forms interface to ProtoRPC services.

This webapp application is automatically configured to work with ProtoRPCs
that have a configured protorpc.RegistryService.  This webapp is
automatically added to the registry service URL at <registry-path>/forms
(default is /protorpc/form) when configured using the
service_handlers.service_mapping function.
"""

import os

from .google_imports import template
from .google_imports import webapp


__all__ = ['FormsHandler',
           'ResourceHandler',

           'DEFAULT_REGISTRY_PATH',
          ]

_TEMPLATES_DIR = os.path.join(os.path.dirname(os.path.dirname(__file__)),
                              'static')

_FORMS_TEMPLATE = os.path.join(_TEMPLATES_DIR, 'forms.html')
_METHODS_TEMPLATE = os.path.join(_TEMPLATES_DIR, 'methods.html')

DEFAULT_REGISTRY_PATH = '/protorpc'


class ResourceHandler(webapp.RequestHandler):
  """Serves static resources without needing to add static files to app.yaml."""

  __RESOURCE_MAP = {
    'forms.js': 'text/javascript',
    'jquery-1.4.2.min.js': 'text/javascript',
    'jquery.json-2.2.min.js': 'text/javascript',
  }

  def get(self, relative):
    """Serve known static files.

    If static file is not known, will return 404 to client.

    Response items are cached for 300 seconds.

    Args:
      relative: Name of static file relative to main FormsHandler.
    """
    content_type = self.__RESOURCE_MAP.get(relative, None)
    if not content_type:
      self.response.set_status(404)
      self.response.out.write('Resource not found.')
      return

    path = os.path.join(_TEMPLATES_DIR, relative)
    self.response.headers['Content-Type'] = content_type
    static_file = open(path)
    try:
      contents = static_file.read()
    finally:
      static_file.close()
    self.response.out.write(contents)


class FormsHandler(webapp.RequestHandler):
  """Handler for display HTML/javascript forms of ProtoRPC method calls.

  When accessed with no query parameters, will show a web page that displays
  all services and methods on the associated registry path.  Links on this
  page fill in the service_path and method_name query parameters back to this
  same handler.

  When provided with service_path and method_name parameters will display a
  dynamic form representing the request message for that method.  When sent,
  the form sends a JSON request to the ProtoRPC method and displays the
  response in the HTML page.

  Attribute:
    registry_path: Read-only registry path known by this handler.
  """

  def __init__(self, registry_path=DEFAULT_REGISTRY_PATH):
    """Constructor.

    When configuring a FormsHandler to use with a webapp application do not
    pass the request handler class in directly.  Instead use new_factory to
    ensure that the FormsHandler is created with the correct registry path
    for each request.

    Args:
      registry_path: Absolute path on server where the ProtoRPC RegsitryService
        is located.
    """
    assert registry_path
    self.__registry_path = registry_path

  @property
  def registry_path(self):
    return self.__registry_path

  def get(self):
    """Send forms and method page to user.

    By default, displays a web page listing all services and methods registered
    on the server.  Methods have links to display the actual method form.

    If both parameters are set, will display form for method.

    Query Parameters:
      service_path: Path to service to display method of.  Optional.
      method_name: Name of method to display form for.  Optional.
    """
    params = {'forms_path': self.request.path.rstrip('/'),
              'hostname': self.request.host,
              'registry_path': self.__registry_path,
    }
    service_path = self.request.get('path', None)
    method_name = self.request.get('method', None)

    if service_path and method_name:
      form_template = _METHODS_TEMPLATE
      params['service_path'] = service_path
      params['method_name'] = method_name
    else:
      form_template = _FORMS_TEMPLATE

    self.response.out.write(template.render(form_template, params))

  @classmethod
  def new_factory(cls, registry_path=DEFAULT_REGISTRY_PATH):
    """Construct a factory for use with WSGIApplication.

    This method is called automatically with the correct registry path when
    services are configured via service_handlers.service_mapping.

    Args:
      registry_path: Absolute path on server where the ProtoRPC RegsitryService
        is located.

    Returns:
      Factory function that creates a properly configured FormsHandler instance.
    """
    def forms_factory():
      return cls(registry_path)
    return forms_factory



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for protorpc.protojson."""

__author__ = 'rafek@google.com (Rafe Kaplan)'


import datetime
import imp
import sys
import unittest

from protorpc import message_types
from protorpc import messages
from protorpc import protojson
from protorpc import test_util

try:
    import json
except ImportError:
    import simplejson as json


class CustomField(messages.MessageField):
  """Custom MessageField class."""

  type = int
  message_type = message_types.VoidMessage

  def __init__(self, number, **kwargs):
    super(CustomField, self).__init__(self.message_type, number, **kwargs)

  def value_to_message(self, value):
    return self.message_type()


class MyMessage(messages.Message):
  """Test message containing various types."""

  class Color(messages.Enum):

    RED = 1
    GREEN = 2
    BLUE = 3

  class Nested(messages.Message):

    nested_value = messages.StringField(1)

  a_string = messages.StringField(2)
  an_integer = messages.IntegerField(3)
  a_float = messages.FloatField(4)
  a_boolean = messages.BooleanField(5)
  an_enum = messages.EnumField(Color, 6)
  a_nested = messages.MessageField(Nested, 7)
  a_repeated = messages.IntegerField(8, repeated=True)
  a_repeated_float = messages.FloatField(9, repeated=True)
  a_datetime = message_types.DateTimeField(10)
  a_repeated_datetime = message_types.DateTimeField(11, repeated=True)
  a_custom = CustomField(12)
  a_repeated_custom = CustomField(13, repeated=True)


class ModuleInterfaceTest(test_util.ModuleInterfaceTest,
                          test_util.TestCase):

  MODULE = protojson


# TODO(rafek): Convert this test to the compliance test in test_util.
class ProtojsonTest(test_util.TestCase,
                    test_util.ProtoConformanceTestBase):
  """Test JSON encoding and decoding."""

  PROTOLIB = protojson

  def CompareEncoded(self, expected_encoded, actual_encoded):
    """JSON encoding will be laundered to remove string differences."""
    self.assertEquals(json.loads(expected_encoded),
                      json.loads(actual_encoded))

  encoded_empty_message = '{}'

  encoded_partial = """{
    "double_value": 1.23,
    "int64_value": -100000000000,
    "int32_value": 1020,
    "string_value": "a string",
    "enum_value": "VAL2"
  }
  """

  encoded_full = """{
    "double_value": 1.23,
    "float_value": -2.5,
    "int64_value": -100000000000,
    "uint64_value": 102020202020,
    "int32_value": 1020,
    "bool_value": true,
    "string_value": "a string\u044f",
    "bytes_value": "YSBieXRlc//+",
    "enum_value": "VAL2"
  }
  """

  encoded_repeated = """{
    "double_value": [1.23, 2.3],
    "float_value": [-2.5, 0.5],
    "int64_value": [-100000000000, 20],
    "uint64_value": [102020202020, 10],
    "int32_value": [1020, 718],
    "bool_value": [true, false],
    "string_value": ["a string\u044f", "another string"],
    "bytes_value": ["YSBieXRlc//+", "YW5vdGhlciBieXRlcw=="],
    "enum_value": ["VAL2", "VAL1"]
  }
  """

  encoded_nested = """{
    "nested": {
      "a_value": "a string"
    }
  }
  """

  encoded_repeated_nested = """{
    "repeated_nested": [{"a_value": "a string"},
                        {"a_value": "another string"}]
  }
  """

  unexpected_tag_message = '{"unknown": "value"}'

  encoded_default_assigned = '{"a_value": "a default"}'

  encoded_nested_empty = '{"nested": {}}'

  encoded_repeated_nested_empty = '{"repeated_nested": [{}, {}]}'

  encoded_extend_message = '{"int64_value": [400, 50, 6000]}'

  encoded_string_types = '{"string_value": "Latin"}'

  encoded_invalid_enum = '{"enum_value": "undefined"}'

  def testConvertIntegerToFloat(self):
    """Test that integers passed in to float fields are converted.

    This is necessary because JSON outputs integers for numbers with 0 decimals.
    """
    message = protojson.decode_message(MyMessage, '{"a_float": 10}')

    self.assertTrue(isinstance(message.a_float, float))
    self.assertEquals(10.0, message.a_float)

  def testConvertStringToNumbers(self):
    """Test that strings passed to integer fields are converted."""
    message = protojson.decode_message(MyMessage,
                                       """{"an_integer": "10",
                                           "a_float": "3.5",
                                           "a_repeated": ["1", "2"],
                                           "a_repeated_float": ["1.5", "2", 10]
                                           }""")

    self.assertEquals(MyMessage(an_integer=10,
                                a_float=3.5,
                                a_repeated=[1, 2],
                                a_repeated_float=[1.5, 2.0, 10.0]),
                      message)

  def testWrongTypeAssignment(self):
    """Test when wrong type is assigned to a field."""
    self.assertRaises(messages.ValidationError,
                      protojson.decode_message,
                      MyMessage, '{"a_string": 10}')
    self.assertRaises(messages.ValidationError,
                      protojson.decode_message,
                      MyMessage, '{"an_integer": 10.2}')
    self.assertRaises(messages.ValidationError,
                      protojson.decode_message,
                      MyMessage, '{"an_integer": "10.2"}')

  def testNumericEnumeration(self):
    """Test that numbers work for enum values."""
    message = protojson.decode_message(MyMessage, '{"an_enum": 2}')

    expected_message = MyMessage()
    expected_message.an_enum = MyMessage.Color.GREEN

    self.assertEquals(expected_message, message)

  def testNumericEnumerationNegativeTest(self):
    """Test with an invalid number for the enum value."""
    self.assertRaisesRegexp(
        messages.DecodeError,
        'Invalid enum value "89"',
        protojson.decode_message,
        MyMessage,
        '{"an_enum": 89}')

  def testAlphaEnumeration(self):
    """Test that alpha enum values work."""
    message = protojson.decode_message(MyMessage, '{"an_enum": "RED"}')

    expected_message = MyMessage()
    expected_message.an_enum = MyMessage.Color.RED

    self.assertEquals(expected_message, message)

  def testAlphaEnumerationNegativeTest(self):
    """The alpha enum value is invalid."""
    self.assertRaisesRegexp(
        messages.DecodeError,
        'Invalid enum value "IAMINVALID"',
        protojson.decode_message,
        MyMessage,
        '{"an_enum": "IAMINVALID"}')

  def testEnumerationNegativeTestWithEmptyString(self):
    """The enum value is an empty string."""
    self.assertRaisesRegexp(
        messages.DecodeError,
        'Invalid enum value ""',
        protojson.decode_message,
        MyMessage,
        '{"an_enum": ""}')

  def testNullValues(self):
    """Test that null values overwrite existing values."""
    self.assertEquals(MyMessage(),
                      protojson.decode_message(MyMessage,
                                               ('{"an_integer": null,'
                                                ' "a_nested": null,'
                                                ' "an_enum": null'
                                                '}')))

  def testEmptyList(self):
    """Test that empty lists are ignored."""
    self.assertEquals(MyMessage(),
                      protojson.decode_message(MyMessage,
                                               '{"a_repeated": []}'))

  def testNotJSON(self):
    """Test error when string is not valid JSON."""
    self.assertRaises(ValueError,
                      protojson.decode_message, MyMessage, '{this is not json}')

  def testDoNotEncodeStrangeObjects(self):
    """Test trying to encode a strange object.

    The main purpose of this test is to complete coverage.  It ensures that
    the default behavior of the JSON encoder is preserved when someone tries to
    serialized an unexpected type.
    """
    class BogusObject(object):

      def check_initialized(self):
        pass

    self.assertRaises(TypeError,
                      protojson.encode_message,
                      BogusObject())

  def testMergeEmptyString(self):
    """Test merging the empty or space only string."""
    message = protojson.decode_message(test_util.OptionalMessage, '')
    self.assertEquals(test_util.OptionalMessage(), message)

    message = protojson.decode_message(test_util.OptionalMessage, ' ')
    self.assertEquals(test_util.OptionalMessage(), message)

  def testMeregeInvalidEmptyMessage(self):
    self.assertRaisesWithRegexpMatch(messages.ValidationError,
                                     'Message NestedMessage is missing '
                                     'required field a_value',
                                     self.PROTOLIB.decode_message,
                                     test_util.NestedMessage,
                                     '')

  def testProtojsonUnrecognizedFieldName(self):
    """Test that unrecognized fields are saved and can be accessed."""
    decoded = protojson.decode_message(MyMessage,
                                       ('{"an_integer": 1, "unknown_val": 2}'))
    self.assertEquals(decoded.an_integer, 1)
    self.assertEquals(1, len(decoded.all_unrecognized_fields()))
    self.assertEquals('unknown_val', decoded.all_unrecognized_fields()[0])
    self.assertEquals((2, messages.Variant.INT64),
                      decoded.get_unrecognized_field_info('unknown_val'))

  def testProtojsonUnrecognizedFieldNumber(self):
    """Test that unrecognized fields are saved and can be accessed."""
    decoded = protojson.decode_message(
        MyMessage,
        '{"an_integer": 1, "1001": "unknown", "-123": "negative", '
        '"456_mixed": 2}')
    self.assertEquals(decoded.an_integer, 1)
    self.assertEquals(3, len(decoded.all_unrecognized_fields()))
    self.assertTrue(1001 in decoded.all_unrecognized_fields())
    self.assertEquals(('unknown', messages.Variant.STRING),
                      decoded.get_unrecognized_field_info(1001))
    self.assertTrue('-123' in decoded.all_unrecognized_fields())
    self.assertEquals(('negative', messages.Variant.STRING),
                      decoded.get_unrecognized_field_info('-123'))
    self.assertTrue('456_mixed' in decoded.all_unrecognized_fields())
    self.assertEquals((2, messages.Variant.INT64),
                      decoded.get_unrecognized_field_info('456_mixed'))

  def testProtojsonUnrecognizedNull(self):
    """Test that unrecognized fields that are None are skipped."""
    decoded = protojson.decode_message(
        MyMessage,
        '{"an_integer": 1, "unrecognized_null": null}')
    self.assertEquals(decoded.an_integer, 1)
    self.assertEquals(decoded.all_unrecognized_fields(), [])

  def testUnrecognizedFieldVariants(self):
    """Test that unrecognized fields are mapped to the right variants."""
    for encoded, expected_variant in (
        ('{"an_integer": 1, "unknown_val": 2}', messages.Variant.INT64),
        ('{"an_integer": 1, "unknown_val": 2.0}', messages.Variant.DOUBLE),
        ('{"an_integer": 1, "unknown_val": "string value"}',
         messages.Variant.STRING),
        ('{"an_integer": 1, "unknown_val": [1, 2, 3]}', messages.Variant.INT64),
        ('{"an_integer": 1, "unknown_val": [1, 2.0, 3]}',
         messages.Variant.DOUBLE),
        ('{"an_integer": 1, "unknown_val": [1, "foo", 3]}',
         messages.Variant.STRING),
        ('{"an_integer": 1, "unknown_val": true}', messages.Variant.BOOL)):
      decoded = protojson.decode_message(MyMessage, encoded)
      self.assertEquals(decoded.an_integer, 1)
      self.assertEquals(1, len(decoded.all_unrecognized_fields()))
      self.assertEquals('unknown_val', decoded.all_unrecognized_fields()[0])
      _, decoded_variant = decoded.get_unrecognized_field_info('unknown_val')
      self.assertEquals(expected_variant, decoded_variant)

  def testDecodeDateTime(self):
    for datetime_string, datetime_vals in (
        ('2012-09-30T15:31:50.262', (2012, 9, 30, 15, 31, 50, 262000)),
        ('2012-09-30T15:31:50', (2012, 9, 30, 15, 31, 50, 0))):
      message = protojson.decode_message(
          MyMessage, '{"a_datetime": "%s"}' % datetime_string)
      expected_message = MyMessage(
          a_datetime=datetime.datetime(*datetime_vals))

      self.assertEquals(expected_message, message)

  def testDecodeInvalidDateTime(self):
    self.assertRaises(messages.DecodeError, protojson.decode_message,
                      MyMessage, '{"a_datetime": "invalid"}')

  def testEncodeDateTime(self):
    for datetime_string, datetime_vals in (
        ('2012-09-30T15:31:50.262000', (2012, 9, 30, 15, 31, 50, 262000)),
        ('2012-09-30T15:31:50.262123', (2012, 9, 30, 15, 31, 50, 262123)),
        ('2012-09-30T15:31:50', (2012, 9, 30, 15, 31, 50, 0))):
      decoded_message = protojson.encode_message(
          MyMessage(a_datetime=datetime.datetime(*datetime_vals)))
      expected_decoding = '{"a_datetime": "%s"}' % datetime_string
      self.CompareEncoded(expected_decoding, decoded_message)

  def testDecodeRepeatedDateTime(self):
    message = protojson.decode_message(
        MyMessage,
        '{"a_repeated_datetime": ["2012-09-30T15:31:50.262", '
        '"2010-01-21T09:52:00", "2000-01-01T01:00:59.999999"]}')
    expected_message = MyMessage(
        a_repeated_datetime=[
            datetime.datetime(2012, 9, 30, 15, 31, 50, 262000),
            datetime.datetime(2010, 1, 21, 9, 52),
            datetime.datetime(2000, 1, 1, 1, 0, 59, 999999)])

    self.assertEquals(expected_message, message)

  def testDecodeCustom(self):
    message = protojson.decode_message(MyMessage, '{"a_custom": 1}')
    self.assertEquals(MyMessage(a_custom=1), message)

  def testDecodeInvalidCustom(self):
    self.assertRaises(messages.ValidationError, protojson.decode_message,
                      MyMessage, '{"a_custom": "invalid"}')

  def testEncodeCustom(self):
    decoded_message = protojson.encode_message(MyMessage(a_custom=1))
    self.CompareEncoded('{"a_custom": 1}', decoded_message)

  def testDecodeRepeatedCustom(self):
    message = protojson.decode_message(
        MyMessage, '{"a_repeated_custom": [1, 2, 3]}')
    self.assertEquals(MyMessage(a_repeated_custom=[1, 2, 3]), message)

  def testDecodeBadBase64BytesField(self):
    """Test decoding improperly encoded base64 bytes value."""
    self.assertRaisesWithRegexpMatch(
        messages.DecodeError,
        'Base64 decoding error: Incorrect padding',
        protojson.decode_message,
        test_util.OptionalMessage,
        '{"bytes_value": "abcdefghijklmnopq"}')


class CustomProtoJson(protojson.ProtoJson):

  def encode_field(self, field, value):
    return '{encoded}' + value

  def decode_field(self, field, value):
    return '{decoded}' + value


class CustomProtoJsonTest(test_util.TestCase):
  """Tests for serialization overriding functionality."""

  def setUp(self):
    self.protojson = CustomProtoJson()

  def testEncode(self):
    self.assertEqual(u'{"a_string": "{encoded}xyz"}',
                     self.protojson.encode_message(MyMessage(a_string=u'xyz')))

  def testDecode(self):
    self.assertEqual(
        MyMessage(a_string=u'{decoded}xyz'),
        self.protojson.decode_message(MyMessage, u'{"a_string": "xyz"}'))

  def testDecodeEmptyMessage(self):
    self.assertEqual(
        MyMessage(a_string=u'{decoded}'),
        self.protojson.decode_message(MyMessage, u'{"a_string": ""}'))

  def testDefault(self):
    self.assertTrue(protojson.ProtoJson.get_default(),
                    protojson.ProtoJson.get_default())

    instance = CustomProtoJson()
    protojson.ProtoJson.set_default(instance)
    self.assertTrue(instance is protojson.ProtoJson.get_default())


class InvalidJsonModule(object):
  pass


class ValidJsonModule(object):
  class JSONEncoder(object):
    pass


class TestJsonDependencyLoading(test_util.TestCase):
  """Test loading various implementations of json."""

  def get_import(self):
    """Get __import__ method.

    Returns:
      The current __import__ method.
    """
    if isinstance(__builtins__, dict):
      return __builtins__['__import__']
    else:
      return __builtins__.__import__

  def set_import(self, new_import):
    """Set __import__ method.

    Args:
      new_import: Function to replace __import__.
    """
    if isinstance(__builtins__, dict):
      __builtins__['__import__'] = new_import
    else:
      __builtins__.__import__ = new_import

  def setUp(self):
    """Save original import function."""
    self.simplejson = sys.modules.pop('simplejson', None)
    self.json = sys.modules.pop('json', None)
    self.original_import = self.get_import()
    def block_all_jsons(name, *args, **kwargs):
      if 'json' in name:
        if name in sys.modules:
          module = sys.modules[name]
          module.name = name
          return module
        raise ImportError('Unable to find %s' % name)
      else:
        return self.original_import(name, *args, **kwargs)
    self.set_import(block_all_jsons)

  def tearDown(self):
    """Restore original import functions and any loaded modules."""

    def reset_module(name, module):
      if module:
        sys.modules[name] = module
      else:
        sys.modules.pop(name, None)
    reset_module('simplejson', self.simplejson)
    reset_module('json', self.json)
    imp.reload(protojson)

  def testLoadProtojsonWithValidJsonModule(self):
    """Test loading protojson module with a valid json dependency."""
    sys.modules['json'] = ValidJsonModule

    # This will cause protojson to reload with the default json module
    # instead of simplejson.
    imp.reload(protojson)
    self.assertEquals('json', protojson.json.name)

  def testLoadProtojsonWithSimplejsonModule(self):
    """Test loading protojson module with simplejson dependency."""
    sys.modules['simplejson'] = ValidJsonModule

    # This will cause protojson to reload with the default json module
    # instead of simplejson.
    imp.reload(protojson)
    self.assertEquals('simplejson', protojson.json.name)

  def testLoadProtojsonWithInvalidJsonModule(self):
    """Loading protojson module with an invalid json defaults to simplejson."""
    sys.modules['json'] = InvalidJsonModule
    sys.modules['simplejson'] = ValidJsonModule

    # Ignore bad module and default back to simplejson.
    imp.reload(protojson)
    self.assertEquals('simplejson', protojson.json.name)

  def testLoadProtojsonWithInvalidJsonModuleAndNoSimplejson(self):
    """Loading protojson module with invalid json and no simplejson."""
    sys.modules['json'] = InvalidJsonModule

    # Bad module without simplejson back raises errors.
    self.assertRaisesWithRegexpMatch(
        ImportError,
        'json library "json" is not compatible with ProtoRPC',
        imp.reload,
        protojson)

  def testLoadProtojsonWithNoJsonModules(self):
    """Loading protojson module with invalid json and no simplejson."""
    # No json modules raise the first exception.
    self.assertRaisesWithRegexpMatch(
        ImportError,
        'Unable to find json',
        imp.reload,
        protojson)


if __name__ == '__main__':
  unittest.main()



"""Dynamically decide from where to import other SDK modules.

All other protorpc.webapp code should import other SDK modules from
this module. If necessary, add new imports here (in both places).
"""

__author__ = 'yey@google.com (Ye Yuan)'

# pylint: disable=g-import-not-at-top
# pylint: disable=unused-import

import os
import sys

try:
  from google.appengine import ext
  normal_environment = True
except ImportError:
  normal_environment = False


if normal_environment:
  from google.appengine.ext import webapp
  from google.appengine.ext.webapp import util as webapp_util
  from google.appengine.ext.webapp import template



#!/usr/bin/env python
#
# Copyright 2011 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""WSGI utility library tests."""
import six
from six.moves import filter

__author__ = 'rafe@google.com (Rafe Kaplan)'


import six.moves.http_client
import unittest

from protorpc import test_util
from protorpc import util
from protorpc import webapp_test_util
from protorpc.wsgi import util as wsgi_util

APP1 = wsgi_util.static_page('App1')
APP2 = wsgi_util.static_page('App2')
NOT_FOUND = wsgi_util.error(six.moves.http_client.NOT_FOUND)


class WsgiTestBase(webapp_test_util.WebServerTestBase):

  server_thread = None

  def CreateWsgiApplication(self):
    return None

  def DoHttpRequest(self,
                    path='/',
                    content=None,
                    content_type='text/plain; charset=utf-8',
                    headers=None):
    connection = six.moves.http_client.HTTPConnection('localhost', self.port)
    if content is None:
      method = 'GET'
    else:
      method = 'POST'
    headers = {'content=type': content_type}
    headers.update(headers)
    connection.request(method, path, content, headers)
    response = connection.getresponse()

    not_date_or_server = lambda header: header[0] not in ('date', 'server')
    headers = list(filter(not_date_or_server, response.getheaders()))

    return response.status, response.reason, response.read(), dict(headers)


class StaticPageBase(WsgiTestBase):

  def testDefault(self):
    default_page = wsgi_util.static_page()
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasContent(self):
    default_page = wsgi_util.static_page('my content')
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('my content', content)
    self.assertEqual({'content-length': str(len('my content')),
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasContentType(self):
    default_page = wsgi_util.static_page(content_type='text/plain')
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/plain',
                      },
                      headers)

  def testHasStatus(self):
    default_page = wsgi_util.static_page(status='400 Not Good Request')
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(400, status)
    self.assertEqual('Not Good Request', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasStatusInt(self):
    default_page = wsgi_util.static_page(status=401)
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(401, status)
    self.assertEqual('Unauthorized', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasStatusUnknown(self):
    default_page = wsgi_util.static_page(status=909)
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(909, status)
    self.assertEqual('Unknown Error', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasStatusTuple(self):
    default_page = wsgi_util.static_page(status=(500, 'Bad Thing'))
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(500, status)
    self.assertEqual('Bad Thing', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testHasHeaders(self):
    default_page = wsgi_util.static_page(headers=[('x', 'foo'),
                                                  ('a', 'bar'),
                                                  ('z', 'bin')])
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                       'x': 'foo',
                       'a': 'bar',
                       'z': 'bin',
                      },
                      headers)

  def testHeadersUnicodeSafe(self):
    default_page = wsgi_util.static_page(headers=[('x', 'foo')])
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                       'x': 'foo',
                      },
                      headers)
    self.assertTrue(isinstance(headers['x'], str))

  def testHasHeadersDict(self):
    default_page = wsgi_util.static_page(headers={'x': 'foo',
                                                  'a': 'bar',
                                                  'z': 'bin'})
    self.ResetServer(default_page)
    status, reason, content, headers = self.DoHttpRequest()
    self.assertEqual(200, status)
    self.assertEqual('OK', reason)
    self.assertEqual('', content)
    self.assertEqual({'content-length': '0',
                       'content-type': 'text/html; charset=utf-8',
                       'x': 'foo',
                       'a': 'bar',
                       'z': 'bin',
                      },
                      headers)


class FirstFoundTest(WsgiTestBase):

  def testEmptyConfiguration(self):
    self.ResetServer(wsgi_util.first_found([]))
    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.NOT_FOUND, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.NOT_FOUND], status_text)
    self.assertEqual(util.pad_string(six.moves.http_client.responses[six.moves.http_client.NOT_FOUND]),
                      content)
    self.assertEqual({'content-length': '512',
                       'content-type': 'text/plain; charset=utf-8',
                      },
                      headers)

  def testOneApp(self):
    self.ResetServer(wsgi_util.first_found([APP1]))

    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.OK, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.OK], status_text)
    self.assertEqual('App1', content)
    self.assertEqual({'content-length': '4',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testIterator(self):
    self.ResetServer(wsgi_util.first_found(iter([APP1])))

    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.OK, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.OK], status_text)
    self.assertEqual('App1', content)
    self.assertEqual({'content-length': '4',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

    # Do request again to make sure iterator was properly copied.
    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.OK, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.OK], status_text)
    self.assertEqual('App1', content)
    self.assertEqual({'content-length': '4',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testTwoApps(self):
    self.ResetServer(wsgi_util.first_found([APP1, APP2]))

    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.OK, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.OK], status_text)
    self.assertEqual('App1', content)
    self.assertEqual({'content-length': '4',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testFirstNotFound(self):
    self.ResetServer(wsgi_util.first_found([NOT_FOUND, APP2]))

    status, status_text, content, headers = self.DoHttpRequest('/')
    self.assertEqual(six.moves.http_client.OK, status)
    self.assertEqual(six.moves.http_client.responses[six.moves.http_client.OK], status_text)
    self.assertEqual('App2', content)
    self.assertEqual({'content-length': '4',
                       'content-type': 'text/html; charset=utf-8',
                      },
                      headers)

  def testOnlyNotFound(self):
    def current_error(environ, start_response):
      """The variable current_status is defined in loop after ResetServer."""
      headers = [('content-type', 'text/plain')]
      status_line = '%03d Whatever' % current_status
      start_response(status_line, headers)
      return []

    self.ResetServer(wsgi_util.first_found([current_error, APP2]))

    statuses_to_check = sorted(httplib.responses.keys())
    # 100, 204 and 304 have slightly different expectations, so they are left
    # out of this test in order to keep the code simple.
    for dont_check in (100, 200, 204, 304, 404):
      statuses_to_check.remove(dont_check)
    for current_status in statuses_to_check:
      status, status_text, content, headers = self.DoHttpRequest('/')
      self.assertEqual(current_status, status)
      self.assertEqual('Whatever', status_text)


if __name__ == '__main__':
  unittest.main()



"""Dynamically decide from where to import other SDK modules.

All other protorpc code should import other SDK modules from
this module. If necessary, add new imports here (in both places).
"""

__author__ = 'yey@google.com (Ye Yuan)'

# pylint: disable=g-import-not-at-top
# pylint: disable=unused-import

from ._google.net.proto import ProtocolBuffer



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Stand-alone implementation of in memory protocol messages.

Public Classes:
  Enum: Represents an enumerated type.
  Variant: Hint for wire format to determine how to serialize.
  Message: Base class for user defined messages.
  IntegerField: Field for integer values.
  FloatField: Field for float values.
  BooleanField: Field for boolean values.
  BytesField: Field for binary string values.
  StringField: Field for UTF-8 string values.
  MessageField: Field for other message type values.
  EnumField: Field for enumerated type values.

Public Exceptions (indentation indications class hierarchy):
  EnumDefinitionError: Raised when enumeration is incorrectly defined.
  FieldDefinitionError: Raised when field is incorrectly defined.
    InvalidVariantError: Raised when variant is not compatible with field type.
    InvalidDefaultError: Raised when default is not compatiable with field.
    InvalidNumberError: Raised when field number is out of range or reserved.
  MessageDefinitionError: Raised when message is incorrectly defined.
    DuplicateNumberError: Raised when field has duplicate number with another.
  ValidationError: Raised when a message or field is not valid.
  DefinitionNotFoundError: Raised when definition not found.
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'


import types
import weakref

from . import util

__all__ = ['MAX_ENUM_VALUE',
           'MAX_FIELD_NUMBER',
           'FIRST_RESERVED_FIELD_NUMBER',
           'LAST_RESERVED_FIELD_NUMBER',

           'Enum',
           'Field',
           'FieldList',
           'Variant',
           'Message',
           'IntegerField',
           'FloatField',
           'BooleanField',
           'BytesField',
           'StringField',
           'MessageField',
           'EnumField',
           'find_definition',

           'Error',
           'DecodeError',
           'EncodeError',
           'EnumDefinitionError',
           'FieldDefinitionError',
           'InvalidVariantError',
           'InvalidDefaultError',
           'InvalidNumberError',
           'MessageDefinitionError',
           'DuplicateNumberError',
           'ValidationError',
           'DefinitionNotFoundError',
          ]


# TODO(rafek): Add extended module test to ensure all exceptions
# in services extends Error.
Error = util.Error


class EnumDefinitionError(Error):
  """Enumeration definition error."""


class FieldDefinitionError(Error):
  """Field definition error."""


class InvalidVariantError(FieldDefinitionError):
  """Invalid variant provided to field."""


class InvalidDefaultError(FieldDefinitionError):
  """Invalid default provided to field."""


class InvalidNumberError(FieldDefinitionError):
  """Invalid number provided to field."""


class MessageDefinitionError(Error):
  """Message definition error."""


class DuplicateNumberError(Error):
  """Duplicate number assigned to field."""


class DefinitionNotFoundError(Error):
  """Raised when definition is not found."""


class DecodeError(Error):
  """Error found decoding message from encoded form."""


class EncodeError(Error):
  """Error found when encoding message."""


class ValidationError(Error):
  """Invalid value for message error."""

  def __str__(self):
    """Prints string with field name if present on exception."""
    message = Error.__str__(self)
    try:
      field_name = self.field_name
    except AttributeError:
      return message
    else:
      return message


# Attributes that are reserved by a class definition that
# may not be used by either Enum or Message class definitions.
_RESERVED_ATTRIBUTE_NAMES = frozenset(
    ['__module__', '__doc__', '__qualname__'])

_POST_INIT_FIELD_ATTRIBUTE_NAMES = frozenset(
    ['name',
     '_message_definition',
     '_MessageField__type',
     '_EnumField__type',
     '_EnumField__resolved_default'])

_POST_INIT_ATTRIBUTE_NAMES = frozenset(
    ['_message_definition'])

# Maximum enumeration value as defined by the protocol buffers standard.
# All enum values must be less than or equal to this value.
MAX_ENUM_VALUE = (2 ** 29) - 1

# Maximum field number as defined by the protocol buffers standard.
# All field numbers must be less than or equal to this value.
MAX_FIELD_NUMBER = (2 ** 29) - 1

# Field numbers between 19000 and 19999 inclusive are reserved by the
# protobuf protocol and may not be used by fields.
FIRST_RESERVED_FIELD_NUMBER = 19000
LAST_RESERVED_FIELD_NUMBER = 19999


class _DefinitionClass(type):
  """Base meta-class used for definition meta-classes.

  The Enum and Message definition classes share some basic functionality.
  Both of these classes may be contained by a Message definition.  After
  initialization, neither class may have attributes changed
  except for the protected _message_definition attribute, and that attribute
  may change only once.
  """

  __initialized = False

  def __init__(cls, name, bases, dct):
    """Constructor."""
    type.__init__(cls, name, bases, dct)
    # Base classes may never be initialized.
    if cls.__bases__ != (object,):
      cls.__initialized = True

  def message_definition(cls):
    """Get outer Message definition that contains this definition.

    Returns:
      Containing Message definition if definition is contained within one,
      else None.
    """
    try:
      return cls._message_definition()
    except AttributeError:
      return None

  def __setattr__(cls, name, value):
    """Overridden so that cannot set variables on definition classes after init.

    Setting attributes on a class must work during the period of initialization
    to set the enumation value class variables and build the name/number maps.
    Once __init__ has set the __initialized flag to True prohibits setting any
    more values on the class.  The class is in effect frozen.

    Args:
      name: Name of value to set.
      value: Value to set.
    """
    if cls.__initialized and name not in _POST_INIT_ATTRIBUTE_NAMES:
      raise AttributeError('May not change values: %s' % name)
    else:
      type.__setattr__(cls, name, value)

  def __delattr__(cls, name):
    """Overridden so that cannot delete varaibles on definition classes."""
    raise TypeError('May not delete attributes on definition class')

  def definition_name(cls):
    """Helper method for creating definition name.

    Names will be generated to include the classes package name, scope (if the
    class is nested in another definition) and class name.

    By default, the package name for a definition is derived from its module
    name.  However, this value can be overriden by placing a 'package' attribute
    in the module that contains the definition class.  For example:

      package = 'some.alternate.package'

      class MyMessage(Message):
        ...

      >>> MyMessage.definition_name()
      some.alternate.package.MyMessage

    Returns:
      Dot-separated fully qualified name of definition.
    """
    outer_definition_name = cls.outer_definition_name()
    if outer_definition_name is None:
      return six.text_type(cls.__name__)
    else:
      return u'%s.%s' % (outer_definition_name, cls.__name__)

  def outer_definition_name(cls):
    """Helper method for creating outer definition name.

    Returns:
      If definition is nested, will return the outer definitions name, else the
      package name.
    """
    outer_definition = cls.message_definition()
    if not outer_definition:
      return util.get_package_for_module(cls.__module__)
    else:
      return outer_definition.definition_name()

  def definition_package(cls):
    """Helper method for creating creating the package of a definition.

    Returns:
      Name of package that definition belongs to.
    """
    outer_definition = cls.message_definition()
    if not outer_definition:
      return util.get_package_for_module(cls.__module__)
    else:
      return outer_definition.definition_package()


class _EnumClass(_DefinitionClass):
  """Meta-class used for defining the Enum base class.

  Meta-class enables very specific behavior for any defined Enum
  class.  All attributes defined on an Enum sub-class must be integers.
  Each attribute defined on an Enum sub-class is translated
  into an instance of that sub-class, with the name of the attribute
  as its name, and the number provided as its value.  It also ensures
  that only one level of Enum class hierarchy is possible.  In other
  words it is not possible to delcare sub-classes of sub-classes of
  Enum.

  This class also defines some functions in order to restrict the
  behavior of the Enum class and its sub-classes.  It is not possible
  to change the behavior of the Enum class in later classes since
  any new classes may be defined with only integer values, and no methods.
  """

  def __init__(cls, name, bases, dct):
    # Can only define one level of sub-classes below Enum.
    if not (bases == (object,) or bases == (Enum,)):
      raise EnumDefinitionError('Enum type %s may only inherit from Enum' %
                                (name,))

    cls.__by_number = {}
    cls.__by_name = {}

    # Enum base class does not need to be initialized or locked.
    if bases != (object,):
      # Replace integer with number.
      for attribute, value in dct.items():

        # Module will be in every enum class.
        if attribute in _RESERVED_ATTRIBUTE_NAMES:
          continue

        # Reject anything that is not an int.
        if not isinstance(value, six.integer_types):
          raise EnumDefinitionError(
              'May only use integers in Enum definitions.  Found: %s = %s' %
              (attribute, value))

        # Protocol buffer standard recommends non-negative values.
        # Reject negative values.
        if value < 0:
          raise EnumDefinitionError(
              'Must use non-negative enum values.  Found: %s = %d' %
              (attribute, value))

        if value > MAX_ENUM_VALUE:
          raise EnumDefinitionError(
              'Must use enum values less than or equal %d.  Found: %s = %d' %
              (MAX_ENUM_VALUE, attribute, value))

        if value in cls.__by_number:
          raise EnumDefinitionError(
              'Value for %s = %d is already defined: %s' %
              (attribute, value, cls.__by_number[value].name))

        # Create enum instance and list in new Enum type.
        instance = object.__new__(cls)
        cls.__init__(instance, attribute, value)
        cls.__by_name[instance.name] = instance
        cls.__by_number[instance.number] = instance
        setattr(cls, attribute, instance)

    _DefinitionClass.__init__(cls, name, bases, dct)

  def __iter__(cls):
    """Iterate over all values of enum.

    Yields:
      Enumeration instances of the Enum class in arbitrary order.
    """
    return iter(cls.__by_number.values())

  def names(cls):
    """Get all names for Enum.

    Returns:
      An iterator for names of the enumeration in arbitrary order.
    """
    return cls.__by_name.keys()

  def numbers(cls):
    """Get all numbers for Enum.

    Returns:
      An iterator for all numbers of the enumeration in arbitrary order.
    """
    return cls.__by_number.keys()

  def lookup_by_name(cls, name):
    """Look up Enum by name.

    Args:
      name: Name of enum to find.

    Returns:
      Enum sub-class instance of that value.
    """
    return cls.__by_name[name]

  def lookup_by_number(cls, number):
    """Look up Enum by number.

    Args:
      number: Number of enum to find.

    Returns:
      Enum sub-class instance of that value.
    """
    return cls.__by_number[number]

  def __len__(cls):
    return len(cls.__by_name)


class Enum(six.with_metaclass(_EnumClass, object)):
  """Base class for all enumerated types."""

  __slots__ = set(('name', 'number'))

  def __new__(cls, index):
    """Acts as look-up routine after class is initialized.

    The purpose of overriding __new__ is to provide a way to treat
    Enum subclasses as casting types, similar to how the int type
    functions.  A program can pass a string or an integer and this
    method with "convert" that value in to an appropriate Enum instance.

    Args:
      index: Name or number to look up.  During initialization
        this is always the name of the new enum value.

    Raises:
      TypeError: When an inappropriate index value is passed provided.
    """
    # If is enum type of this class, return it.
    if isinstance(index, cls):
      return index

    # If number, look up by number.
    if isinstance(index, six.integer_types):
      try:
        return cls.lookup_by_number(index)
      except KeyError:
        pass

    # If name, look up by name.
    if isinstance(index, six.string_types):
      try:
        return cls.lookup_by_name(index)
      except KeyError:
        pass

    raise TypeError('No such value for %s in Enum %s' %
                    (index, cls.__name__))

  def __init__(self, name, number=None):
    """Initialize new Enum instance.

    Since this should only be called during class initialization any
    calls that happen after the class is frozen raises an exception.
    """
    # Immediately return if __init__ was called after _Enum.__init__().
    # It means that casting operator version of the class constructor
    # is being used.
    if getattr(type(self), '_DefinitionClass__initialized'):
      return
    object.__setattr__(self, 'name', name)
    object.__setattr__(self, 'number', number)

  def __setattr__(self, name, value):
    raise TypeError('May not change enum values')

  def __str__(self):
    return self.name

  def __int__(self):
    return self.number

  def __repr__(self):
    return '%s(%s, %d)' % (type(self).__name__, self.name, self.number)

  def __reduce__(self):
    """Enable pickling.

    Returns:
      A 2-tuple containing the class and __new__ args to be used for restoring
      a pickled instance.
    """
    return self.__class__, (self.number,)

  def __cmp__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return cmp(self.number, other.number)
    return NotImplemented

  def __lt__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number < other.number
    return NotImplemented

  def __le__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number <= other.number
    return NotImplemented

  def __eq__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number == other.number
    return NotImplemented

  def __ne__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number != other.number
    return NotImplemented

  def __ge__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number >= other.number
    return NotImplemented

  def __gt__(self, other):
    """Order is by number."""
    if isinstance(other, type(self)):
      return self.number > other.number
    return NotImplemented

  def __hash__(self):
    """Hash by number."""
    return hash(self.number)

  @classmethod
  def to_dict(cls):
    """Make dictionary version of enumerated class.

    Dictionary created this way can be used with def_num.

    Returns:
      A dict (name) -> number
    """
    return dict((item.name, item.number) for item in iter(cls))

  @staticmethod
  def def_enum(dct, name):
    """Define enum class from dictionary.

    Args:
      dct: Dictionary of enumerated values for type.
      name: Name of enum.
    """
    return type(name, (Enum,), dct)


# TODO(rafek): Determine to what degree this enumeration should be compatible
# with FieldDescriptor.Type in:
#
#   http://code.google.com/p/protobuf/source/browse/trunk/src/google/protobuf/descriptor.proto
class Variant(Enum):
  """Wire format variant.

  Used by the 'protobuf' wire format to determine how to transmit
  a single piece of data.  May be used by other formats.

  See: http://code.google.com/apis/protocolbuffers/docs/encoding.html

  Values:
    DOUBLE: 64-bit floating point number.
    FLOAT: 32-bit floating point number.
    INT64: 64-bit signed integer.
    UINT64: 64-bit unsigned integer.
    INT32: 32-bit signed integer.
    BOOL: Boolean value (True or False).
    STRING: String of UTF-8 encoded text.
    MESSAGE: Embedded message as byte string.
    BYTES: String of 8-bit bytes.
    UINT32: 32-bit unsigned integer.
    ENUM: Enum value as integer.
    SINT32: 32-bit signed integer.  Uses "zig-zag" encoding.
    SINT64: 64-bit signed integer.  Uses "zig-zag" encoding.
  """
  DOUBLE   = 1
  FLOAT    = 2
  INT64    = 3
  UINT64   = 4
  INT32    = 5
  BOOL     = 8
  STRING   = 9
  MESSAGE  = 11
  BYTES    = 12
  UINT32   = 13
  ENUM     = 14
  SINT32   = 17
  SINT64   = 18


class _MessageClass(_DefinitionClass):
  """Meta-class used for defining the Message base class.

  For more details about Message classes, see the Message class docstring.
  Information contained there may help understanding this class.

  Meta-class enables very specific behavior for any defined Message
  class.  All attributes defined on an Message sub-class must be field
  instances, Enum class definitions or other Message class definitions.  Each
  field attribute defined on an Message sub-class is added to the set of
  field definitions and the attribute is translated in to a slot.  It also
  ensures that only one level of Message class hierarchy is possible.  In other
  words it is not possible to declare sub-classes of sub-classes of
  Message.

  This class also defines some functions in order to restrict the
  behavior of the Message class and its sub-classes.  It is not possible
  to change the behavior of the Message class in later classes since
  any new classes may be defined with only field, Enums and Messages, and
  no methods.
  """

  def __new__(cls, name, bases, dct):
    """Create new Message class instance.

    The __new__ method of the _MessageClass type is overridden so as to
    allow the translation of Field instances to slots.
    """
    by_number = {}
    by_name = {}

    variant_map = {}

    if bases != (object,):
      # Can only define one level of sub-classes below Message.
      if bases != (Message,):
        raise MessageDefinitionError(
            'Message types may only inherit from Message')

      enums = []
      messages = []
      # Must not use iteritems because this loop will change the state of dct.
      for key, field in dct.items():

        if key in _RESERVED_ATTRIBUTE_NAMES:
          continue

        if isinstance(field, type) and issubclass(field, Enum):
          enums.append(key)
          continue

        if (isinstance(field, type) and
            issubclass(field, Message) and
            field is not Message):
          messages.append(key)
          continue

        # Reject anything that is not a field.
        if type(field) is Field or not isinstance(field, Field):
          raise MessageDefinitionError(
              'May only use fields in message definitions.  Found: %s = %s' %
              (key, field))

        if field.number in by_number:
          raise DuplicateNumberError(
              'Field with number %d declared more than once in %s' %
              (field.number, name))

        field.name = key

        # Place in name and number maps.
        by_name[key] = field
        by_number[field.number] = field

      # Add enums if any exist.
      if enums:
        dct['__enums__'] = sorted(enums)

      # Add messages if any exist.
      if messages:
        dct['__messages__'] = sorted(messages)

    dct['_Message__by_number'] = by_number
    dct['_Message__by_name'] = by_name

    return _DefinitionClass.__new__(cls, name, bases, dct)

  def __init__(cls, name, bases, dct):
    """Initializer required to assign references to new class."""
    if bases != (object,):
      for value in dct.values():
        if isinstance(value, _DefinitionClass) and not value is Message:
          value._message_definition = weakref.ref(cls)

      for field in cls.all_fields():
        field._message_definition = weakref.ref(cls)

    _DefinitionClass.__init__(cls, name, bases, dct)


class Message(six.with_metaclass(_MessageClass, object)):
  """Base class for user defined message objects.

  Used to define messages for efficient transmission across network or
  process space.  Messages are defined using the field classes (IntegerField,
  FloatField, EnumField, etc.).

  Messages are more restricted than normal classes in that they may only
  contain field attributes and other Message and Enum definitions.  These
  restrictions are in place because the structure of the Message class is
  intentended to itself be transmitted across network or process space and
  used directly by clients or even other servers.  As such methods and
  non-field attributes could not be transmitted with the structural information
  causing discrepancies between different languages and implementations.

  Initialization and validation:

    A Message object is considered to be initialized if it has all required
    fields and any nested messages are also initialized.

    Calling 'check_initialized' will raise a ValidationException if it is not
    initialized; 'is_initialized' returns a boolean value indicating if it is
    valid.

    Validation automatically occurs when Message objects are created
    and populated.  Validation that a given value will be compatible with
    a field that it is assigned to can be done through the Field instances
    validate() method.  The validate method used on a message will check that
    all values of a message and its sub-messages are valid.  Assingning an
    invalid value to a field will raise a ValidationException.

  Example:

    # Trade type.
    class TradeType(Enum):
      BUY = 1
      SELL = 2
      SHORT = 3
      CALL = 4

    class Lot(Message):
      price = IntegerField(1, required=True)
      quantity = IntegerField(2, required=True)

    class Order(Message):
      symbol = StringField(1, required=True)
      total_quantity = IntegerField(2, required=True)
      trade_type = EnumField(TradeType, 3, required=True)
      lots = MessageField(Lot, 4, repeated=True)
      limit = IntegerField(5)

    order = Order(symbol='GOOG',
                  total_quantity=10,
                  trade_type=TradeType.BUY)

    lot1 = Lot(price=304,
               quantity=7)

    lot2 = Lot(price = 305,
               quantity=3)

    order.lots = [lot1, lot2]

    # Now object is initialized!
    order.check_initialized()
  """

  def __init__(self, **kwargs):
    """Initialize internal messages state.

    Args:
      A message can be initialized via the constructor by passing in keyword
      arguments corresponding to fields.  For example:

        class Date(Message):
          day = IntegerField(1)
          month = IntegerField(2)
          year = IntegerField(3)

      Invoking:

        date = Date(day=6, month=6, year=1911)

      is the same as doing:

        date = Date()
        date.day = 6
        date.month = 6
        date.year = 1911
    """
    # Tag being an essential implementation detail must be private.
    self.__tags = {}
    self.__unrecognized_fields = {}

    assigned = set()
    for name, value in kwargs.items():
      setattr(self, name, value)
      assigned.add(name)

    # initialize repeated fields.
    for field in self.all_fields():
      if field.repeated and field.name not in assigned:
        setattr(self, field.name, [])


  def check_initialized(self):
    """Check class for initialization status.

    Check that all required fields are initialized

    Raises:
      ValidationError: If message is not initialized.
    """
    for name, field in self.__by_name.items():
      value = getattr(self, name)
      if value is None:
        if field.required:
          raise ValidationError("Message %s is missing required field %s" %
                                (type(self).__name__, name))
      else:
        try:
          if (isinstance(field, MessageField) and
              issubclass(field.message_type, Message)):
            if field.repeated:
              for item in value:
                item_message_value = field.value_to_message(item)
                item_message_value.check_initialized()
            else:
              message_value = field.value_to_message(value)
              message_value.check_initialized()
        except ValidationError as err:
          if not hasattr(err, 'message_name'):
            err.message_name = type(self).__name__
          raise

  def is_initialized(self):
    """Get initialization status.

    Returns:
      True if message is valid, else False.
    """
    try:
      self.check_initialized()
    except ValidationError:
      return False
    else:
      return True

  @classmethod
  def all_fields(cls):
    """Get all field definition objects.

    Ordering is arbitrary.

    Returns:
      Iterator over all values in arbitrary order.
    """
    return cls.__by_name.values()

  @classmethod
  def field_by_name(cls, name):
    """Get field by name.

    Returns:
      Field object associated with name.

    Raises:
      KeyError if no field found by that name.
    """
    return cls.__by_name[name]

  @classmethod
  def field_by_number(cls, number):
    """Get field by number.

    Returns:
      Field object associated with number.

    Raises:
      KeyError if no field found by that number.
    """
    return cls.__by_number[number]

  def get_assigned_value(self, name):
    """Get the assigned value of an attribute.

    Get the underlying value of an attribute.  If value has not been set, will
    not return the default for the field.

    Args:
      name: Name of attribute to get.

    Returns:
      Value of attribute, None if it has not been set.
    """
    message_type = type(self)
    try:
      field = message_type.field_by_name(name)
    except KeyError:
      raise AttributeError('Message %s has no field %s' % (
          message_type.__name__, name))
    return self.__tags.get(field.number)

  def reset(self, name):
    """Reset assigned value for field.

    Resetting a field will return it to its default value or None.

    Args:
      name: Name of field to reset.
    """
    message_type = type(self)
    try:
      field = message_type.field_by_name(name)
    except KeyError:
      if name not in message_type.__by_name:
        raise AttributeError('Message %s has no field %s' % (
            message_type.__name__, name))
    if field.repeated:
      self.__tags[field.number] = FieldList(field, [])
    else:
      self.__tags.pop(field.number, None)

  def all_unrecognized_fields(self):
    """Get the names of all unrecognized fields in this message."""
    return list(self.__unrecognized_fields.keys())

  def get_unrecognized_field_info(self, key, value_default=None,
                                  variant_default=None):
    """Get the value and variant of an unknown field in this message.

    Args:
      key: The name or number of the field to retrieve.
      value_default: Value to be returned if the key isn't found.
      variant_default: Value to be returned as variant if the key isn't
        found.

    Returns:
      (value, variant), where value and variant are whatever was passed
      to set_unrecognized_field.
    """
    value, variant = self.__unrecognized_fields.get(key, (value_default,
                                                          variant_default))
    return value, variant

  def set_unrecognized_field(self, key, value, variant):
    """Set an unrecognized field, used when decoding a message.

    Args:
      key: The name or number used to refer to this unknown value.
      value: The value of the field.
      variant: Type information needed to interpret the value or re-encode it.

    Raises:
      TypeError: If the variant is not an instance of messages.Variant.
    """
    if not isinstance(variant, Variant):
      raise TypeError('Variant type %s is not valid.' % variant)
    self.__unrecognized_fields[key] = value, variant

  def __setattr__(self, name, value):
    """Change set behavior for messages.

    Messages may only be assigned values that are fields.

    Does not try to validate field when set.

    Args:
      name: Name of field to assign to.
      value: Value to assign to field.

    Raises:
      AttributeError when trying to assign value that is not a field.
    """
    if name in self.__by_name or name.startswith('_Message__'):
      object.__setattr__(self, name, value)
    else:
      raise AttributeError("May not assign arbitrary value %s "
                           "to message %s" % (name, type(self).__name__))

  def __repr__(self):
    """Make string representation of message.

    Example:

      class MyMessage(messages.Message):
        integer_value = messages.IntegerField(1)
        string_value = messages.StringField(2)

      my_message = MyMessage()
      my_message.integer_value = 42
      my_message.string_value = u'A string'

      print my_message
      >>> <MyMessage
      ...  integer_value: 42
      ...  string_value: u'A string'>

    Returns:
      String representation of message, including the values
      of all fields and repr of all sub-messages.
    """
    body = ['<', type(self).__name__]
    for field in sorted(self.all_fields(),
                        key=lambda f: f.number):
      attribute = field.name
      value = self.get_assigned_value(field.name)
      if value is not None:
        body.append('\n %s: %s' % (attribute, repr(value)))
    body.append('>')
    return ''.join(body)

  def __eq__(self, other):
    """Equality operator.

    Does field by field comparison with other message.  For
    equality, must be same type and values of all fields must be
    equal.

    Messages not required to be initialized for comparison.

    Does not attempt to determine equality for values that have
    default values that are not set.  In other words:

      class HasDefault(Message):

        attr1 = StringField(1, default='default value')

      message1 = HasDefault()
      message2 = HasDefault()
      message2.attr1 = 'default value'

      message1 != message2

    Does not compare unknown values.

    Args:
      other: Other message to compare with.
    """
    # TODO(rafek): Implement "equivalent" which does comparisons
    # taking default values in to consideration.
    if self is other:
      return True

    if type(self) is not type(other):
      return False

    return self.__tags == other.__tags

  def __ne__(self, other):
    """Not equals operator.

    Does field by field comparison with other message.  For
    non-equality, must be different type or any value of a field must be
    non-equal to the same field in the other instance.

    Messages not required to be initialized for comparison.

    Args:
      other: Other message to compare with.
    """
    return not self.__eq__(other)


class FieldList(list):
  """List implementation that validates field values.

  This list implementation overrides all methods that add values in to a list
  in order to validate those new elements.  Attempting to add or set list
  values that are not of the correct type will raise ValidationError.
  """

  def __init__(self, field_instance, sequence):
    """Constructor.

    Args:
      field_instance: Instance of field that validates the list.
      sequence: List or tuple to construct list from.
    """
    if not field_instance.repeated:
      raise FieldDefinitionError('FieldList may only accept repeated fields')
    self.__field = field_instance
    self.__field.validate(sequence)
    list.__init__(self, sequence)

  def __getstate__(self):
    """Enable pickling.

    The assigned field instance can't be pickled if it belongs to a Message
    definition (message_definition uses a weakref), so the Message class and
    field number are returned in that case.

    Returns:
      A 3-tuple containing:
        - The field instance, or None if it belongs to a Message class.
        - The Message class that the field instance belongs to, or None.
        - The field instance number of the Message class it belongs to, or None.
    """
    message_class = self.__field.message_definition()
    if message_class is None:
      return self.__field, None, None
    else:
      return None, message_class, self.__field.number

  def __setstate__(self, state):
    """Enable unpickling.

    Args:
      state: A 3-tuple containing:
        - The field instance, or None if it belongs to a Message class.
        - The Message class that the field instance belongs to, or None.
        - The field instance number of the Message class it belongs to, or None.
    """
    field_instance, message_class, number = state
    if field_instance is None:
      self.__field = message_class.field_by_number(number)
    else:
      self.__field = field_instance

  @property
  def field(self):
    """Field that validates list."""
    return self.__field

  def __setslice__(self, i, j, sequence):
    """Validate slice assignment to list."""
    self.__field.validate(sequence)
    list.__setslice__(self, i, j, sequence)

  def __setitem__(self, index, value):
    """Validate item assignment to list."""
    if isinstance(index, slice):
        self.__field.validate(value)
    else:
        self.__field.validate_element(value)
    list.__setitem__(self, index, value)

  def append(self, value):
    """Validate item appending to list."""
    self.__field.validate_element(value)
    return list.append(self, value)

  def extend(self, sequence):
    """Validate extension of list."""
    self.__field.validate(sequence)
    return list.extend(self, sequence)

  def insert(self, index, value):
    """Validate item insertion to list."""
    self.__field.validate_element(value)
    return list.insert(self, index, value)


class _FieldMeta(type):

  def __init__(cls, name, bases, dct):
    getattr(cls, '_Field__variant_to_type').update(
      (variant, cls) for variant in dct.get('VARIANTS', []))
    type.__init__(cls, name, bases, dct)


# TODO(rafek): Prevent additional field subclasses.
class Field(six.with_metaclass(_FieldMeta, object)):

  __initialized = False
  __variant_to_type = {}

  @util.positional(2)
  def __init__(self,
               number,
               required=False,
               repeated=False,
               variant=None,
               default=None):
    """Constructor.

    The required and repeated parameters are mutually exclusive.  Setting both
    to True will raise a FieldDefinitionError.

    Sub-class Attributes:
      Each sub-class of Field must define the following:
        VARIANTS: Set of variant types accepted by that field.
        DEFAULT_VARIANT: Default variant type if not specified in constructor.

    Args:
      number: Number of field.  Must be unique per message class.
      required: Whether or not field is required.  Mutually exclusive with
        'repeated'.
      repeated: Whether or not field is repeated.  Mutually exclusive with
        'required'.
      variant: Wire-format variant hint.
      default: Default value for field if not found in stream.

    Raises:
      InvalidVariantError when invalid variant for field is provided.
      InvalidDefaultError when invalid default for field is provided.
      FieldDefinitionError when invalid number provided or mutually exclusive
        fields are used.
      InvalidNumberError when the field number is out of range or reserved.
    """
    if not isinstance(number, int) or not 1 <= number <= MAX_FIELD_NUMBER:
      raise InvalidNumberError('Invalid number for field: %s\n'
                               'Number must be 1 or greater and %d or less' %
                               (number, MAX_FIELD_NUMBER))

    if FIRST_RESERVED_FIELD_NUMBER <= number <= LAST_RESERVED_FIELD_NUMBER:
      raise InvalidNumberError('Tag number %d is a reserved number.\n'
                               'Numbers %d to %d are reserved' %
                               (number, FIRST_RESERVED_FIELD_NUMBER,
                                LAST_RESERVED_FIELD_NUMBER))

    if repeated and required:
      raise FieldDefinitionError('Cannot set both repeated and required')

    if variant is None:
      variant = self.DEFAULT_VARIANT

    if repeated and default is not None:
      raise FieldDefinitionError('Repeated fields may not have defaults')

    if variant not in self.VARIANTS:
      raise InvalidVariantError(
          'Invalid variant: %s\nValid variants for %s are %r' %
          (variant, type(self).__name__, sorted(self.VARIANTS)))

    self.number = number
    self.required = required
    self.repeated = repeated
    self.variant = variant

    if default is not None:
      try:
        self.validate_default(default)
      except ValidationError as err:
        try:
          name = self.name
        except AttributeError:
          # For when raising error before name initialization.
          raise InvalidDefaultError('Invalid default value for %s: %r: %s' %
                                    (self.__class__.__name__, default, err))
        else:
          raise InvalidDefaultError('Invalid default value for field %s: '
                                    '%r: %s' % (name, default, err))

    self.__default = default
    self.__initialized = True

  def __setattr__(self, name, value):
    """Setter overidden to prevent assignment to fields after creation.

    Args:
      name: Name of attribute to set.
      value: Value to assign.
    """
    # Special case post-init names.  They need to be set after constructor.
    if name in _POST_INIT_FIELD_ATTRIBUTE_NAMES:
      object.__setattr__(self, name, value)
      return

    # All other attributes must be set before __initialized.
    if not self.__initialized:
      # Not initialized yet, allow assignment.
      object.__setattr__(self, name, value)
    else:
      raise AttributeError('Field objects are read-only')

  def __set__(self, message_instance, value):
    """Set value on message.

    Args:
      message_instance: Message instance to set value on.
      value: Value to set on message.
    """
    # Reaches in to message instance directly to assign to private tags.
    if value is None:
      if self.repeated:
        raise ValidationError(
          'May not assign None to repeated field %s' % self.name)
      else:
        message_instance._Message__tags.pop(self.number, None)
    else:
      if self.repeated:
        value = FieldList(self, value)
      else:
        value = self.validate(value)
      message_instance._Message__tags[self.number] = value

  def __get__(self, message_instance, message_class):
    if message_instance is None:
      return self

    result = message_instance._Message__tags.get(self.number)
    if result is None:
      return self.default
    else:
      return result

  def validate_element(self, value):
    """Validate single element of field.

    This is different from validate in that it is used on individual
    values of repeated fields.

    Args:
      value: Value to validate.

    Returns:
      The value casted in the expectes type.

    Raises:
      ValidationError if value is not expected type.
    """
    if not isinstance(value, self.type):
      # Authorize in values as float
      if isinstance(value, six.integer_types) and self.type == float:
        return float(value)

      if value is None:
        if self.required:
          raise ValidationError('Required field is missing')
      else:
        try:
          name = self.name
        except AttributeError:
          raise ValidationError('Expected type %s for %s, '
                                'found %s (type %s)' %
                                (self.type, self.__class__.__name__,
                                 value, type(value)))
        else:
          raise ValidationError('Expected type %s for field %s, '
                                'found %s (type %s)' %
                                (self.type, name, value, type(value)))
    return value

  def __validate(self, value, validate_element):
    """Internal validation function.

    Validate an internal value using a function to validate individual elements.

    Args:
      value: Value to validate.
      validate_element: Function to use to validate individual elements.

    Raises:
      ValidationError if value is not expected type.
    """
    if not self.repeated:
      return validate_element(value)
    else:
      # Must be a list or tuple, may not be a string.
      if isinstance(value, (list, tuple)):
        result = []
        for element in value:
          if element is None:
            try:
              name = self.name
            except AttributeError:
              raise ValidationError('Repeated values for %s '
                                    'may not be None' % self.__class__.__name__)
            else:
              raise ValidationError('Repeated values for field %s '
                                    'may not be None' % name)
          result.append(validate_element(element))
        return result
      elif value is not None:
        try:
          name = self.name
        except AttributeError:
          raise ValidationError('%s is repeated. Found: %s' % (
            self.__class__.__name__, value))
        else:
          raise ValidationError('Field %s is repeated. Found: %s' % (name,
                                                                     value))
    return value

  def validate(self, value):
    """Validate value assigned to field.

    Args:
      value: Value to validate.

    Returns:
      the value eventually casted in the correct type.

    Raises:
      ValidationError if value is not expected type.
    """
    return self.__validate(value, self.validate_element)

  def validate_default_element(self, value):
    """Validate value as assigned to field default field.

    Some fields may allow for delayed resolution of default types necessary
    in the case of circular definition references.  In this case, the default
    value might be a place holder that is resolved when needed after all the
    message classes are defined.

    Args:
      value: Default value to validate.

    Returns:
      the value eventually casted in the correct type.

    Raises:
      ValidationError if value is not expected type.
    """
    return self.validate_element(value)

  def validate_default(self, value):
    """Validate default value assigned to field.

    Args:
      value: Value to validate.

    Returns:
      the value eventually casted in the correct type.

    Raises:
      ValidationError if value is not expected type.
    """
    return self.__validate(value, self.validate_default_element)

  def message_definition(self):
    """Get Message definition that contains this Field definition.

    Returns:
      Containing Message definition for Field.  Will return None if for
      some reason Field is defined outside of a Message class.
    """
    try:
      return self._message_definition()
    except AttributeError:
      return None

  @property
  def default(self):
    """Get default value for field."""
    return self.__default

  @classmethod
  def lookup_field_type_by_variant(cls, variant):
    return cls.__variant_to_type[variant]


class IntegerField(Field):
  """Field definition for integer values."""

  VARIANTS = frozenset([Variant.INT32,
                        Variant.INT64,
                        Variant.UINT32,
                        Variant.UINT64,
                        Variant.SINT32,
                        Variant.SINT64,
                       ])

  DEFAULT_VARIANT = Variant.INT64

  type = six.integer_types


class FloatField(Field):
  """Field definition for float values."""

  VARIANTS = frozenset([Variant.FLOAT,
                        Variant.DOUBLE,
                       ])

  DEFAULT_VARIANT = Variant.DOUBLE

  type = float


class BooleanField(Field):
  """Field definition for boolean values."""

  VARIANTS = frozenset([Variant.BOOL])

  DEFAULT_VARIANT = Variant.BOOL

  type = bool


class BytesField(Field):
  """Field definition for byte string values."""

  VARIANTS = frozenset([Variant.BYTES])

  DEFAULT_VARIANT = Variant.BYTES

  type = bytes


class StringField(Field):
  """Field definition for unicode string values."""

  VARIANTS = frozenset([Variant.STRING])

  DEFAULT_VARIANT = Variant.STRING

  type = six.text_type

  def validate_element(self, value):
    """Validate StringField allowing for str and unicode.

    Raises:
      ValidationError if a str value is not 7-bit ascii.
    """
    # If value is str is it considered valid.  Satisfies "required=True".
    if isinstance(value, bytes):
      try:
        six.text_type(value, 'ascii')
      except UnicodeDecodeError as err:
        try:
          name = self.name
        except AttributeError:
          validation_error = ValidationError(
            'Field encountered non-ASCII string %r: %s' % (value,
                                                           err))
        else:
          validation_error = ValidationError(
            'Field %s encountered non-ASCII string %r: %s' % (self.name,
                                                              value,
                                                              err))
          validation_error.field_name = self.name
        raise validation_error
    else:
      return super(StringField, self).validate_element(value)


class MessageField(Field):
  """Field definition for sub-message values.

  Message fields contain instance of other messages.  Instances stored
  on messages stored on message fields  are considered to be owned by
  the containing message instance and should not be shared between
  owning instances.

  Message fields must be defined to reference a single type of message.
  Normally message field are defined by passing the referenced message
  class in to the constructor.

  It is possible to define a message field for a type that does not yet
  exist by passing the name of the message in to the constructor instead
  of a message class.  Resolution of the actual type of the message is
  deferred until it is needed, for example, during message verification.
  Names provided to the constructor must refer to a class within the same
  python module as the class that is using it.  Names refer to messages
  relative to the containing messages scope.  For example, the two fields
  of OuterMessage refer to the same message type:

    class Outer(Message):

      inner_relative = MessageField('Inner', 1)
      inner_absolute = MessageField('Outer.Inner', 2)

      class Inner(Message):
        ...

  When resolving an actual type, MessageField will traverse the entire
  scope of nested messages to match a message name.  This makes it easy
  for siblings to reference siblings:

    class Outer(Message):

      class Inner(Message):

        sibling = MessageField('Sibling', 1)

      class Sibling(Message):
        ...
  """

  VARIANTS = frozenset([Variant.MESSAGE])

  DEFAULT_VARIANT = Variant.MESSAGE

  @util.positional(3)
  def __init__(self,
               message_type,
               number,
               required=False,
               repeated=False,
               variant=None):
    """Constructor.

    Args:
      message_type: Message type for field.  Must be subclass of Message.
      number: Number of field.  Must be unique per message class.
      required: Whether or not field is required.  Mutually exclusive to
        'repeated'.
      repeated: Whether or not field is repeated.  Mutually exclusive to
        'required'.
      variant: Wire-format variant hint.

    Raises:
      FieldDefinitionError when invalid message_type is provided.
    """
    valid_type = (isinstance(message_type, six.string_types) or
                  (message_type is not Message and
                   isinstance(message_type, type) and
                   issubclass(message_type, Message)))

    if not valid_type:
      raise FieldDefinitionError('Invalid message class: %s' % message_type)

    if isinstance(message_type, six.string_types):
      self.__type_name = message_type
      self.__type = None
    else:
      self.__type = message_type

    super(MessageField, self).__init__(number,
                                       required=required,
                                       repeated=repeated,
                                       variant=variant)

  def __set__(self, message_instance, value):
    """Set value on message.

    Args:
      message_instance: Message instance to set value on.
      value: Value to set on message.
    """
    message_type = self.type
    if isinstance(message_type, type) and issubclass(message_type, Message):
      if self.repeated:
        if value and isinstance(value, (list, tuple)):
          value = [(message_type(**v) if isinstance(v, dict) else v)
                   for v in value]
      elif isinstance(value, dict):
        value = message_type(**value)
    super(MessageField, self).__set__(message_instance, value)

  @property
  def type(self):
    """Message type used for field."""
    if self.__type is None:
      message_type = find_definition(self.__type_name, self.message_definition())
      if not (message_type is not Message and
              isinstance(message_type, type) and
              issubclass(message_type, Message)):
        raise FieldDefinitionError('Invalid message class: %s' % message_type)
      self.__type = message_type
    return self.__type

  @property
  def message_type(self):
    """Underlying message type used for serialization.

    Will always be a sub-class of Message.  This is different from type
    which represents the python value that message_type is mapped to for
    use by the user.
    """
    return self.type

  def value_from_message(self, message):
    """Convert a message to a value instance.

    Used by deserializers to convert from underlying messages to
    value of expected user type.

    Args:
      message: A message instance of type self.message_type.

    Returns:
      Value of self.message_type.
    """
    if not isinstance(message, self.message_type):
      raise DecodeError('Expected type %s, got %s: %r' %
                        (self.message_type.__name__,
                         type(message).__name__,
                         message))
    return message

  def value_to_message(self, value):
    """Convert a value instance to a message.

    Used by serializers to convert Python user types to underlying
    messages for transmission.

    Args:
      value: A value of type self.type.

    Returns:
      An instance of type self.message_type.
    """
    if not isinstance(value, self.type):
      raise EncodeError('Expected type %s, got %s: %r' %
                        (self.type.__name__,
                         type(value).__name__,
                         value))
    return value


class EnumField(Field):
  """Field definition for enum values.

  Enum fields may have default values that are delayed until the associated enum
  type is resolved.  This is necessary to support certain circular references.

  For example:

    class Message1(Message):

      class Color(Enum):

        RED = 1
        GREEN = 2
        BLUE = 3

      # This field default value  will be validated when default is accessed.
      animal = EnumField('Message2.Animal', 1, default='HORSE')

    class Message2(Message):

      class Animal(Enum):

        DOG = 1
        CAT = 2
        HORSE = 3

      # This fields default value will be validated right away since Color is
      # already fully resolved.
      color = EnumField(Message1.Color, 1, default='RED')
  """

  VARIANTS = frozenset([Variant.ENUM])

  DEFAULT_VARIANT = Variant.ENUM

  def __init__(self, enum_type, number, **kwargs):
    """Constructor.

    Args:
      enum_type: Enum type for field.  Must be subclass of Enum.
      number: Number of field.  Must be unique per message class.
      required: Whether or not field is required.  Mutually exclusive to
        'repeated'.
      repeated: Whether or not field is repeated.  Mutually exclusive to
        'required'.
      variant: Wire-format variant hint.
      default: Default value for field if not found in stream.

    Raises:
      FieldDefinitionError when invalid enum_type is provided.
    """
    valid_type = (isinstance(enum_type, six.string_types) or
                  (enum_type is not Enum and
                   isinstance(enum_type, type) and
                   issubclass(enum_type, Enum)))

    if not valid_type:
      raise FieldDefinitionError('Invalid enum type: %s' % enum_type)

    if isinstance(enum_type, six.string_types):
      self.__type_name = enum_type
      self.__type = None
    else:
      self.__type = enum_type

    super(EnumField, self).__init__(number, **kwargs)

  def validate_default_element(self, value):
    """Validate default element of Enum field.

    Enum fields allow for delayed resolution of default values when the type
    of the field has not been resolved.  The default value of a field may be
    a string or an integer.  If the Enum type of the field has been resolved,
    the default value is validated against that type.

    Args:
      value: Value to validate.

    Raises:
      ValidationError if value is not expected message type.
    """
    if isinstance(value, (six.string_types, six.integer_types)):
      # Validation of the value does not happen for delayed resolution
      # enumerated types.  Ignore if type is not yet resolved.
      if self.__type:
        self.__type(value)
      return

    return super(EnumField, self).validate_default_element(value)

  @property
  def type(self):
    """Enum type used for field."""
    if self.__type is None:
      found_type = find_definition(self.__type_name, self.message_definition())
      if not (found_type is not Enum and
              isinstance(found_type, type) and
              issubclass(found_type, Enum)):
        raise FieldDefinitionError('Invalid enum type: %s' % found_type)

      self.__type = found_type
    return self.__type

  @property
  def default(self):
    """Default for enum field.

    Will cause resolution of Enum type and unresolved default value.
    """
    try:
      return self.__resolved_default
    except AttributeError:
      resolved_default = super(EnumField, self).default
      if isinstance(resolved_default, (six.string_types, six.integer_types)):
        resolved_default = self.type(resolved_default)
      self.__resolved_default = resolved_default
      return self.__resolved_default


@util.positional(2)
def find_definition(name, relative_to=None, importer=__import__):
  """Find definition by name in module-space.

  The find algorthm will look for definitions by name relative to a message
  definition or by fully qualfied name.  If no definition is found relative
  to the relative_to parameter it will do the same search against the container
  of relative_to.  If relative_to is a nested Message, it will search its
  message_definition().  If that message has no message_definition() it will
  search its module.  If relative_to is a module, it will attempt to look for
  the containing module and search relative to it.  If the module is a top-level
  module, it will look for the a message using a fully qualified name.  If
  no message is found then, the search fails and DefinitionNotFoundError is
  raised.

  For example, when looking for any definition 'foo.bar.ADefinition' relative to
  an actual message definition abc.xyz.SomeMessage:

    find_definition('foo.bar.ADefinition', SomeMessage)

  It is like looking for the following fully qualified names:

    abc.xyz.SomeMessage. foo.bar.ADefinition
    abc.xyz. foo.bar.ADefinition
    abc. foo.bar.ADefinition
    foo.bar.ADefinition

  When resolving the name relative to Message definitions and modules, the
  algorithm searches any Messages or sub-modules found in its path.
  Non-Message values are not searched.

  A name that begins with '.' is considered to be a fully qualified name.  The
  name is always searched for from the topmost package.  For example, assume
  two message types:

    abc.xyz.SomeMessage
    xyz.SomeMessage

  Searching for '.xyz.SomeMessage' relative to 'abc' will resolve to
  'xyz.SomeMessage' and not 'abc.xyz.SomeMessage'.  For this kind of name,
  the relative_to parameter is effectively ignored and always set to None.

  For more information about package name resolution, please see:

    http://code.google.com/apis/protocolbuffers/docs/proto.html#packages

  Args:
    name: Name of definition to find.  May be fully qualified or relative name.
    relative_to: Search for definition relative to message definition or module.
      None will cause a fully qualified name search.
    importer: Import function to use for resolving modules.

  Returns:
    Enum or Message class definition associated with name.

  Raises:
    DefinitionNotFoundError if no definition is found in any search path.
  """
  # Check parameters.
  if not (relative_to is None or
          isinstance(relative_to, types.ModuleType) or
          isinstance(relative_to, type) and issubclass(relative_to, Message)):
    raise TypeError('relative_to must be None, Message definition or module.  '
                    'Found: %s' % relative_to)

  name_path = name.split('.')

  # Handle absolute path reference.
  if not name_path[0]:
    relative_to = None
    name_path = name_path[1:]

  def search_path():
    """Performs a single iteration searching the path from relative_to.

    This is the function that searches up the path from a relative object.

      fully.qualified.object . relative.or.nested.Definition
                               ---------------------------->
                                                  ^
                                                  |
                            this part of search --+

    Returns:
      Message or Enum at the end of name_path, else None.
    """
    next = relative_to
    for node in name_path:
      # Look for attribute first.
      attribute = getattr(next, node, None)

      if attribute is not None:
        next = attribute
      else:
        # If module, look for sub-module.
        if next is None or isinstance(next, types.ModuleType):
          if next is None:
            module_name = node
          else:
            module_name = '%s.%s' % (next.__name__, node)

          try:
            fromitem = module_name.split('.')[-1]
            next = importer(module_name, '', '', [str(fromitem)])
          except ImportError:
            return None
        else:
          return None

      if (not isinstance(next, types.ModuleType) and
          not (isinstance(next, type) and
               issubclass(next, (Message, Enum)))):
        return None

    return next

  while True:
    found = search_path()
    if isinstance(found, type) and issubclass(found, (Enum, Message)):
      return found
    else:
      # Find next relative_to to search against.
      #
      #   fully.qualified.object . relative.or.nested.Definition
      #   <---------------------
      #           ^
      #           |
      #   does this part of search
      if relative_to is None:
        # Fully qualified search was done.  Nothing found.  Fail.
        raise DefinitionNotFoundError('Could not find definition for %s'
                                      % (name,))
      else:
        if isinstance(relative_to, types.ModuleType):
          # Find parent module.
          module_path = relative_to.__name__.split('.')[:-1]
          if not module_path:
            relative_to = None
          else:
            # Should not raise ImportError.  If it does... weird and
            # unexepected.  Propagate.
            relative_to = importer(
              '.'.join(module_path), '', '', [module_path[-1]])
        elif (isinstance(relative_to, type) and
              issubclass(relative_to, Message)):
          parent = relative_to.message_definition()
          if parent is None:
            last_module_name = relative_to.__module__.split('.')[-1]
            relative_to = importer(
              relative_to.__module__, '', '', [last_module_name])
          else:
            relative_to = parent



#!/usr/bin/env python
#
# Copyright 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Services descriptor definitions.

Contains message definitions and functions for converting
service classes into transmittable message format.

Describing an Enum instance, Enum class, Field class or Message class will
generate an appropriate descriptor object that describes that class.
This message can itself be used to transmit information to clients wishing
to know the description of an enum value, enum, field or message without
needing to download the source code.  This format is also compatible with
other, non-Python languages.

The descriptors are modeled to be binary compatible with:

  http://code.google.com/p/protobuf/source/browse/trunk/src/google/protobuf/descriptor.proto

NOTE: The names of types and fields are not always the same between these
descriptors and the ones defined in descriptor.proto.  This was done in order
to make source code files that use these descriptors easier to read.  For
example, it is not necessary to prefix TYPE to all the values in
FieldDescriptor.Variant as is done in descriptor.proto FieldDescriptorProto.Type.

Example:

  class Pixel(messages.Message):

    x = messages.IntegerField(1, required=True)
    y = messages.IntegerField(2, required=True)

    color = messages.BytesField(3)

  # Describe Pixel class using message descriptor.
  fields = []

  field = FieldDescriptor()
  field.name = 'x'
  field.number = 1
  field.label = FieldDescriptor.Label.REQUIRED
  field.variant = FieldDescriptor.Variant.INT64
  fields.append(field)

  field = FieldDescriptor()
  field.name = 'y'
  field.number = 2
  field.label = FieldDescriptor.Label.REQUIRED
  field.variant = FieldDescriptor.Variant.INT64
  fields.append(field)

  field = FieldDescriptor()
  field.name = 'color'
  field.number = 3
  field.label = FieldDescriptor.Label.OPTIONAL
  field.variant = FieldDescriptor.Variant.BYTES
  fields.append(field)

  message = MessageDescriptor()
  message.name = 'Pixel'
  message.fields = fields

  # Describing is the equivalent of building the above message.
  message == describe_message(Pixel)

Public Classes:
  EnumValueDescriptor: Describes Enum values.
  EnumDescriptor: Describes Enum classes.
  FieldDescriptor: Describes field instances.
  FileDescriptor: Describes a single 'file' unit.
  FileSet: Describes a collection of file descriptors.
  MessageDescriptor: Describes Message classes.
  MethodDescriptor: Describes a method of a service.
  ServiceDescriptor: Describes a services.

Public Functions:
  describe_enum_value: Describe an individual enum-value.
  describe_enum: Describe an Enum class.
  describe_field: Describe a Field definition.
  describe_file: Describe a 'file' unit from a Python module or object.
  describe_file_set: Describe a file set from a list of modules or objects.
  describe_message: Describe a Message definition.
  describe_method: Describe a Method definition.
  describe_service: Describe a Service definition.
"""
import six

__author__ = 'rafek@google.com (Rafe Kaplan)'

import codecs
import types

from . import messages
from . import util


__all__ = ['EnumDescriptor',
           'EnumValueDescriptor',
           'FieldDescriptor',
           'MessageDescriptor',
           'MethodDescriptor',
           'FileDescriptor',
           'FileSet',
           'ServiceDescriptor',
           'DescriptorLibrary',

           'describe_enum',
           'describe_enum_value',
           'describe_field',
           'describe_message',
           'describe_method',
           'describe_file',
           'describe_file_set',
           'describe_service',
           'describe',
           'import_descriptor_loader',
          ]


# NOTE: MessageField is missing because message fields cannot have
# a default value at this time.
# TODO(rafek): Support default message values.
#
# Map to functions that convert default values of fields of a given type
# to a string.  The function must return a value that is compatible with
# FieldDescriptor.default_value and therefore a unicode string.
_DEFAULT_TO_STRING_MAP = {
    messages.IntegerField: six.text_type,
    messages.FloatField: six.text_type,
    messages.BooleanField: lambda value: value and u'true' or u'false',
    messages.BytesField: lambda value: codecs.escape_encode(value)[0],
    messages.StringField: lambda value: value,
    messages.EnumField: lambda value: six.text_type(value.number),
}

_DEFAULT_FROM_STRING_MAP = {
    messages.IntegerField: int,
    messages.FloatField: float,
    messages.BooleanField: lambda value: value == u'true',
    messages.BytesField: lambda value: codecs.escape_decode(value)[0],
    messages.StringField: lambda value: value,
    messages.EnumField: int,
}


class EnumValueDescriptor(messages.Message):
  """Enum value descriptor.

  Fields:
    name: Name of enumeration value.
    number: Number of enumeration value.
  """

  # TODO(rafek): Why are these listed as optional in descriptor.proto.
  # Harmonize?
  name = messages.StringField(1, required=True)
  number = messages.IntegerField(2,
                                 required=True,
                                 variant=messages.Variant.INT32)


class EnumDescriptor(messages.Message):
  """Enum class descriptor.

  Fields:
    name: Name of Enum without any qualification.
    values: Values defined by Enum class.
  """

  name = messages.StringField(1)
  values = messages.MessageField(EnumValueDescriptor, 2, repeated=True)


class FieldDescriptor(messages.Message):
  """Field definition descriptor.

  Enums:
    Variant: Wire format hint sub-types for field.
    Label: Values for optional, required and repeated fields.

  Fields:
    name: Name of field.
    number: Number of field.
    variant: Variant of field.
    type_name: Type name for message and enum fields.
    default_value: String representation of default value.
  """

  Variant = messages.Variant

  class Label(messages.Enum):
    """Field label."""

    OPTIONAL = 1
    REQUIRED = 2
    REPEATED = 3

  name = messages.StringField(1, required=True)
  number = messages.IntegerField(3,
                                 required=True,
                                 variant=messages.Variant.INT32)
  label = messages.EnumField(Label, 4, default=Label.OPTIONAL)
  variant = messages.EnumField(Variant, 5)
  type_name = messages.StringField(6)

  # For numeric types, contains the original text representation of the value.
  # For booleans, "true" or "false".
  # For strings, contains the default text contents (not escaped in any way).
  # For bytes, contains the C escaped value.  All bytes < 128 are that are
  #   traditionally considered unprintable are also escaped.
  default_value = messages.StringField(7)


class MessageDescriptor(messages.Message):
  """Message definition descriptor.

  Fields:
    name: Name of Message without any qualification.
    fields: Fields defined for message.
    message_types: Nested Message classes defined on message.
    enum_types: Nested Enum classes defined on message.
  """

  name = messages.StringField(1)
  fields = messages.MessageField(FieldDescriptor, 2, repeated=True)

  message_types = messages.MessageField(
    'protorpc.descriptor.MessageDescriptor', 3, repeated=True)
  enum_types = messages.MessageField(EnumDescriptor, 4, repeated=True)


class MethodDescriptor(messages.Message):
  """Service method definition descriptor.

  Fields:
    name: Name of service method.
    request_type: Fully qualified or relative name of request message type.
    response_type: Fully qualified or relative name of response message type.
  """

  name = messages.StringField(1)

  request_type = messages.StringField(2)
  response_type = messages.StringField(3)


class ServiceDescriptor(messages.Message):
  """Service definition descriptor.

  Fields:
    name: Name of Service without any qualification.
    methods: Remote methods of Service.
  """

  name = messages.StringField(1)

  methods = messages.MessageField(MethodDescriptor, 2, repeated=True)


class FileDescriptor(messages.Message):
  """Description of file containing protobuf definitions.

  Fields:
    package: Fully qualified name of package that definitions belong to.
    message_types: Message definitions contained in file.
    enum_types: Enum definitions contained in file.
    service_types: Service definitions contained in file.
  """

  package = messages.StringField(2)

  # TODO(rafek): Add dependency field

  message_types = messages.MessageField(MessageDescriptor, 4, repeated=True)
  enum_types = messages.MessageField(EnumDescriptor, 5, repeated=True)
  service_types = messages.MessageField(ServiceDescriptor, 6, repeated=True)


class FileSet(messages.Message):
  """A collection of FileDescriptors.

  Fields:
    files: Files in file-set.
  """

  files = messages.MessageField(FileDescriptor, 1, repeated=True)


def describe_enum_value(enum_value):
  """Build descriptor for Enum instance.

  Args:
    enum_value: Enum value to provide descriptor for.

  Returns:
    Initialized EnumValueDescriptor instance describing the Enum instance.
  """
  enum_value_descriptor = EnumValueDescriptor()
  enum_value_descriptor.name = six.text_type(enum_value.name)
  enum_value_descriptor.number = enum_value.number
  return enum_value_descriptor


def describe_enum(enum_definition):
  """Build descriptor for Enum class.

  Args:
    enum_definition: Enum class to provide descriptor for.

  Returns:
    Initialized EnumDescriptor instance describing the Enum class.
  """
  enum_descriptor = EnumDescriptor()
  enum_descriptor.name = enum_definition.definition_name().split('.')[-1]

  values = []
  for number in enum_definition.numbers():
    value = enum_definition.lookup_by_number(number)
    values.append(describe_enum_value(value))

  if values:
    enum_descriptor.values = values

  return enum_descriptor


def describe_field(field_definition):
  """Build descriptor for Field instance.

  Args:
    field_definition: Field instance to provide descriptor for.

  Returns:
    Initialized FieldDescriptor instance describing the Field instance.
  """
  field_descriptor = FieldDescriptor()
  field_descriptor.name = field_definition.name
  field_descriptor.number = field_definition.number
  field_descriptor.variant = field_definition.variant

  if isinstance(field_definition, messages.EnumField):
    field_descriptor.type_name = field_definition.type.definition_name()

  if isinstance(field_definition, messages.MessageField):
    field_descriptor.type_name = field_definition.message_type.definition_name()

  if field_definition.default is not None:
    field_descriptor.default_value = _DEFAULT_TO_STRING_MAP[
        type(field_definition)](field_definition.default)

  # Set label.
  if field_definition.repeated:
    field_descriptor.label = FieldDescriptor.Label.REPEATED
  elif field_definition.required:
    field_descriptor.label = FieldDescriptor.Label.REQUIRED
  else:
    field_descriptor.label = FieldDescriptor.Label.OPTIONAL

  return field_descriptor


def describe_message(message_definition):
  """Build descriptor for Message class.

  Args:
    message_definition: Message class to provide descriptor for.

  Returns:
    Initialized MessageDescriptor instance describing the Message class.
  """
  message_descriptor = MessageDescriptor()
  message_descriptor.name = message_definition.definition_name().split('.')[-1]

  fields = sorted(message_definition.all_fields(),
                  key=lambda v: v.number)
  if fields:
    message_descriptor.fields = [describe_field(field) for field in fields]

  try:
    nested_messages = message_definition.__messages__
  except AttributeError:
    pass
  else:
    message_descriptors = []
    for name in nested_messages:
      value = getattr(message_definition, name)
      message_descriptors.append(describe_message(value))

    message_descriptor.message_types = message_descriptors

  try:
    nested_enums = message_definition.__enums__
  except AttributeError:
    pass
  else:
    enum_descriptors = []
    for name in nested_enums:
      value = getattr(message_definition, name)
      enum_descriptors.append(describe_enum(value))

    message_descriptor.enum_types = enum_descriptors

  return message_descriptor


def describe_method(method):
  """Build descriptor for service method.

  Args:
    method: Remote service method to describe.

  Returns:
    Initialized MethodDescriptor instance describing the service method.
  """
  method_info = method.remote
  descriptor = MethodDescriptor()
  descriptor.name = method_info.method.__name__
  descriptor.request_type = method_info.request_type.definition_name()
  descriptor.response_type = method_info.response_type.definition_name()

  return descriptor


def describe_service(service_class):
  """Build descriptor for service.

  Args:
    service_class: Service class to describe.

  Returns:
    Initialized ServiceDescriptor instance describing the service.
  """
  descriptor = ServiceDescriptor()
  descriptor.name = service_class.__name__
  methods = []
  remote_methods = service_class.all_remote_methods()
  for name in sorted(remote_methods.keys()):
    if name == 'get_descriptor':
      continue

    method = remote_methods[name]
    methods.append(describe_method(method))
  if methods:
    descriptor.methods = methods

  return descriptor


def describe_file(module):
  """Build a file from a specified Python module.

  Args:
    module: Python module to describe.

  Returns:
    Initialized FileDescriptor instance describing the module.
  """
  # May not import remote at top of file because remote depends on this
  # file
  # TODO(rafek): Straighten out this dependency.  Possibly move these functions
  # from descriptor to their own module.
  from . import remote

  descriptor = FileDescriptor()
  descriptor.package = util.get_package_for_module(module)

  if not descriptor.package:
    descriptor.package = None

  message_descriptors = []
  enum_descriptors = []
  service_descriptors = []

  # Need to iterate over all top level attributes of the module looking for
  # message, enum and service definitions.  Each definition must be itself
  # described.
  for name in sorted(dir(module)):
    value = getattr(module, name)

    if isinstance(value, type):
      if issubclass(value, messages.Message):
        message_descriptors.append(describe_message(value))

      elif issubclass(value, messages.Enum):
        enum_descriptors.append(describe_enum(value))

      elif issubclass(value, remote.Service):
        service_descriptors.append(describe_service(value))

  if message_descriptors:
    descriptor.message_types = message_descriptors

  if enum_descriptors:
    descriptor.enum_types = enum_descriptors

  if service_descriptors:
    descriptor.service_types = service_descriptors

  return descriptor


def describe_file_set(modules):
  """Build a file set from a specified Python modules.

  Args:
    modules: Iterable of Python module to describe.

  Returns:
    Initialized FileSet instance describing the modules.
  """
  descriptor = FileSet()
  file_descriptors = []
  for module in modules:
    file_descriptors.append(describe_file(module))

  if file_descriptors:
    descriptor.files = file_descriptors

  return descriptor


def describe(value):
  """Describe any value as a descriptor.

  Helper function for describing any object with an appropriate descriptor
  object.

  Args:
    value: Value to describe as a descriptor.

  Returns:
    Descriptor message class if object is describable as a descriptor, else
    None.
  """
  from . import remote
  if isinstance(value, types.ModuleType):
    return describe_file(value)
  elif callable(value) and hasattr(value, 'remote'):
    return describe_method(value)
  elif isinstance(value, messages.Field):
    return describe_field(value)
  elif isinstance(value, messages.Enum):
    return describe_enum_value(value)
  elif isinstance(value, type):
    if issubclass(value, messages.Message):
      return describe_message(value)
    elif issubclass(value, messages.Enum):
      return describe_enum(value)
    elif issubclass(value, remote.Service):
      return describe_service(value)
  return None


@util.positional(1)
def import_descriptor_loader(definition_name, importer=__import__):
  """Find objects by importing modules as needed.

  A definition loader is a function that resolves a definition name to a
  descriptor.

  The import finder resolves definitions to their names by importing modules
  when necessary.

  Args:
    definition_name: Name of definition to find.
    importer: Import function used for importing new modules.

  Returns:
    Appropriate descriptor for any describable type located by name.

  Raises:
    DefinitionNotFoundError when a name does not refer to either a definition
    or a module.
  """
  # Attempt to import descriptor as a module.
  if definition_name.startswith('.'):
    definition_name = definition_name[1:]
  if not definition_name.startswith('.'):
    leaf = definition_name.split('.')[-1]
    if definition_name:
      try:
        module = importer(definition_name, '', '', [leaf])
      except ImportError:
        pass
      else:
        return describe(module)

  try:
    # Attempt to use messages.find_definition to find item.
    return describe(messages.find_definition(definition_name,
                                             importer=__import__))
  except messages.DefinitionNotFoundError as err:
    # There are things that find_definition will not find, but if the parent
    # is loaded, its children can be searched for a match.
    split_name = definition_name.rsplit('.', 1)
    if len(split_name) > 1:
      parent, child = split_name
      try:
        parent_definition = import_descriptor_loader(parent, importer=importer)
      except messages.DefinitionNotFoundError:
        # Fall through to original error.
        pass
      else:
        # Check the parent definition for a matching descriptor.
        if isinstance(parent_definition, FileDescriptor):
          search_list = parent_definition.service_types or []
        elif isinstance(parent_definition, ServiceDescriptor):
          search_list = parent_definition.methods or []
        elif isinstance(parent_definition, EnumDescriptor):
          search_list = parent_definition.values or []
        elif isinstance(parent_definition, MessageDescriptor):
          search_list = parent_definition.fields or []
        else:
          search_list = []

        for definition in search_list:
          if definition.name == child:
            return definition

    # Still didn't find.  Reraise original exception.
    raise err


class DescriptorLibrary(object):
  """A descriptor library is an object that contains known definitions.

  A descriptor library contains a cache of descriptor objects mapped by
  definition name.  It contains all types of descriptors except for
  file sets.

  When a definition name is requested that the library does not know about
  it can be provided with a descriptor loader which attempt to resolve the
  missing descriptor.
  """

  @util.positional(1)
  def __init__(self,
               descriptors=None,
               descriptor_loader=import_descriptor_loader):
    """Constructor.

    Args:
      descriptors: A dictionary or dictionary-like object that can be used
        to store and cache descriptors by definition name.
      definition_loader: A function used for resolving missing descriptors.
        The function takes a definition name as its parameter and returns
        an appropriate descriptor.  It may raise DefinitionNotFoundError.
    """
    self.__descriptor_loader = descriptor_loader
    self.__descriptors = descriptors or {}

  def lookup_descriptor(self, definition_name):
    """Lookup descriptor by name.

    Get descriptor from library by name.  If descriptor is not found will
    attempt to find via descriptor loader if provided.

    Args:
      definition_name: Definition name to find.

    Returns:
      Descriptor that describes definition name.

    Raises:
      DefinitionNotFoundError if not descriptor exists for definition name.
    """
    try:
      return self.__descriptors[definition_name]
    except KeyError:
      pass

    if self.__descriptor_loader:
      definition = self.__descriptor_loader(definition_name)
      self.__descriptors[definition_name] = definition
      return definition
    else:  
      raise messages.DefinitionNotFoundError(
        'Could not find definition for %s' % definition_name)

  def lookup_package(self, definition_name):
    """Determines the package name for any definition.

    Determine the package that any definition name belongs to.  May check
    parent for package name and will resolve missing descriptors if provided
    descriptor loader.

    Args:
      definition_name: Definition name to find package for.
    """
    while True:
      descriptor = self.lookup_descriptor(definition_name)
      if isinstance(descriptor, FileDescriptor):
        return descriptor.package
      else:
        index = definition_name.rfind('.')
        if index < 0:
          return None
        definition_name = definition_name[:index]
